package javax.swing;

import java.awt.event.FocusListener;
import java.awt.FontMetrics;
import java.awt.Font;
import java.awt.Cursor;
import javax.accessibility.AccessibleValue;
import javax.accessibility.AccessibleText;
import javax.accessibility.AccessibleAction;
import java.util.Locale;
import javax.accessibility.AccessibleState;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleComponent;
import javax.accessibility.AccessibleTableModelChange;
import javax.accessibility.AccessibleTable;
import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleExtendedTable;
import javax.accessibility.AccessibleSelection;
import java.awt.print.PageFormat;
import java.awt.Graphics;
import java.applet.Applet;
import java.awt.Window;
import java.beans.PropertyChangeEvent;
import sun.reflect.misc.ReflectUtil;
import javax.swing.border.LineBorder;
import java.lang.reflect.Constructor;
import javax.swing.border.EmptyBorder;
import java.text.DateFormat;
import java.text.NumberFormat;
import javax.accessibility.AccessibleContext;
import java.awt.print.Printable;
import java.awt.print.PrinterAbortException;
import sun.swing.PrintingStatus;
import javax.print.attribute.HashPrintRequestAttributeSet;
import java.awt.print.PrinterJob;
import javax.print.PrintService;
import javax.print.attribute.PrintRequestAttributeSet;
import java.text.MessageFormat;
import java.awt.print.PrinterException;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import javax.swing.table.DefaultTableColumnModel;
import java.util.Date;
import javax.swing.table.DefaultTableCellRenderer;
import java.awt.event.KeyEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ChangeEvent;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.event.RowSorterEvent;
import javax.swing.event.TableModelEvent;
import java.util.Enumeration;
import javax.swing.plaf.TableUI;
import java.util.EventObject;
import sun.awt.AWTAccessor;
import java.awt.event.MouseEvent;
import javax.swing.table.TableRowSorter;
import java.awt.Rectangle;
import sun.swing.SwingUtilities2;
import java.awt.Point;
import java.awt.HeadlessException;
import java.awt.GraphicsEnvironment;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import java.awt.KeyboardFocusManager;
import javax.swing.border.Border;
import javax.swing.plaf.UIResource;
import java.awt.Container;
import javax.swing.table.AbstractTableModel;
import java.util.Vector;
import javax.swing.table.DefaultTableModel;
import java.beans.PropertyChangeListener;
import java.util.Hashtable;
import javax.swing.table.TableCellEditor;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Color;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;
import javax.swing.event.RowSorterListener;
import javax.accessibility.Accessible;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableColumnModelListener;
import javax.swing.event.TableModelListener;

public class JTable extends JComponent implements TableModelListener, Scrollable, TableColumnModelListener, ListSelectionListener, CellEditorListener, Accessible, RowSorterListener
{
    private static final String uiClassID = "TableUI";
    public static final int AUTO_RESIZE_OFF = 0;
    public static final int AUTO_RESIZE_NEXT_COLUMN = 1;
    public static final int AUTO_RESIZE_SUBSEQUENT_COLUMNS = 2;
    public static final int AUTO_RESIZE_LAST_COLUMN = 3;
    public static final int AUTO_RESIZE_ALL_COLUMNS = 4;
    protected TableModel dataModel;
    protected TableColumnModel columnModel;
    protected ListSelectionModel selectionModel;
    protected JTableHeader tableHeader;
    protected int rowHeight;
    protected int rowMargin;
    protected Color gridColor;
    protected boolean showHorizontalLines;
    protected boolean showVerticalLines;
    protected int autoResizeMode;
    protected boolean autoCreateColumnsFromModel;
    protected Dimension preferredViewportSize;
    protected boolean rowSelectionAllowed;
    protected boolean cellSelectionEnabled;
    protected transient Component editorComp;
    protected transient TableCellEditor cellEditor;
    protected transient int editingColumn;
    protected transient int editingRow;
    protected transient Hashtable defaultRenderersByColumnClass;
    protected transient Hashtable defaultEditorsByColumnClass;
    protected Color selectionForeground;
    protected Color selectionBackground;
    private SizeSequence rowModel;
    private boolean dragEnabled;
    private boolean surrendersFocusOnKeystroke;
    private PropertyChangeListener editorRemover;
    private boolean columnSelectionAdjusting;
    private boolean rowSelectionAdjusting;
    private Throwable printError;
    private boolean isRowHeightSet;
    private boolean updateSelectionOnSort;
    private transient JTable.SortManager sortManager;
    private boolean ignoreSortChange;
    private boolean sorterChanged;
    private boolean autoCreateRowSorter;
    private boolean fillsViewportHeight;
    private DropMode dropMode;
    private transient JTable.DropLocation dropLocation;
    
    public JTable() {
        this(null, null, null);
    }
    
    public JTable(TableModel tableModel) {
        this(tableModel, null, null);
    }
    
    public JTable(TableModel tableModel, TableColumnModel tableColumnModel) {
        this(tableModel, tableColumnModel, null);
    }
    
    public JTable(TableModel defaultDataModel, TableColumnModel defaultColumnModel, ListSelectionModel defaultSelectionModel) {
        this.editorRemover = null;
        this.dropMode = DropMode.USE_SELECTION;
        this.setLayout(null);
        this.setFocusTraversalKeys(0, JComponent.getManagingFocusForwardTraversalKeys());
        this.setFocusTraversalKeys(1, JComponent.getManagingFocusBackwardTraversalKeys());
        if (defaultColumnModel == null) {
            defaultColumnModel = this.createDefaultColumnModel();
            this.autoCreateColumnsFromModel = true;
        }
        this.setColumnModel(defaultColumnModel);
        if (defaultSelectionModel == null) {
            defaultSelectionModel = this.createDefaultSelectionModel();
        }
        this.setSelectionModel(defaultSelectionModel);
        if (defaultDataModel == null) {
            defaultDataModel = this.createDefaultDataModel();
        }
        this.setModel(defaultDataModel);
        this.initializeLocalVars();
        this.updateUI();
    }
    
    public JTable(int rowCount, int columnCount) {
        this(new DefaultTableModel(rowCount, columnCount));
    }
    
    public JTable(Vector data, Vector columnNames) {
        this(new DefaultTableModel(data, columnNames));
    }
    
    public JTable(final Object[][] array, final Object[] array2) {
        this(new AbstractTableModel() {
            @Override
            public String getColumnName(int n) {
                return array2[n].toString();
            }
            
            @Override
            public int getRowCount() {
                return array.length;
            }
            
            @Override
            public int getColumnCount() {
                return array2.length;
            }
            
            @Override
            public Object getValueAt(int n, int n2) {
                return array[n][n2];
            }
            
            @Override
            public boolean isCellEditable(int n, int n2) {
                return true;
            }
            
            @Override
            public void setValueAt(Object o, int row, int column) {
                array[row][column] = o;
                this.fireTableCellUpdated(row, column);
            }
        });
    }
    
    @Override
    public void addNotify() {
        super.addNotify();
        this.configureEnclosingScrollPane();
    }
    
    protected void configureEnclosingScrollPane() {
        Container unwrappedParent = SwingUtilities.getUnwrappedParent(this);
        if ((unwrappedParent instanceof JViewport)) {
            Container parent = ((JViewport)unwrappedParent).getParent();
            if ((parent instanceof JScrollPane)) {
                JScrollPane scrollPane = (JScrollPane)parent;
                JViewport viewport = scrollPane.getViewport();
                if (viewport == null || SwingUtilities.getUnwrappedView(viewport) != this) {
                    return;
                }
                scrollPane.setColumnHeaderView(this.getTableHeader());
                this.configureEnclosingScrollPaneUI();
            }
        }
    }
    
    private void configureEnclosingScrollPaneUI() {
        Container unwrappedParent = SwingUtilities.getUnwrappedParent(this);
        if ((unwrappedParent instanceof JViewport)) {
            Container parent = ((JViewport)unwrappedParent).getParent();
            if ((parent instanceof JScrollPane)) {
                JScrollPane scrollPane = (JScrollPane)parent;
                JViewport viewport = scrollPane.getViewport();
                if (viewport == null || SwingUtilities.getUnwrappedView(viewport) != this) {
                    return;
                }
                Border border = scrollPane.getBorder();
                if (border == null || border instanceof UIResource) {
                    Border border2 = UIManager.getBorder("Table.scrollPaneBorder");
                    if (border2 != null) {
                        scrollPane.setBorder(border2);
                    }
                }
                Component corner = scrollPane.getCorner("UPPER_TRAILING_CORNER");
                if (corner == null || corner instanceof UIResource) {
                    Component component = null;
                    try {
                        component = (Component)UIManager.get("Table.scrollPaneCornerComponent");
                    }
                    catch (Exception ex) {}
                    scrollPane.setCorner("UPPER_TRAILING_CORNER", component);
                }
            }
        }
    }
    
    @Override
    public void removeNotify() {
        KeyboardFocusManager.getCurrentKeyboardFocusManager().removePropertyChangeListener("permanentFocusOwner", this.editorRemover);
        this.editorRemover = null;
        this.unconfigureEnclosingScrollPane();
        super.removeNotify();
    }
    
    protected void unconfigureEnclosingScrollPane() {
        Container unwrappedParent = SwingUtilities.getUnwrappedParent(this);
        if ((unwrappedParent instanceof JViewport)) {
            Container parent = ((JViewport)unwrappedParent).getParent();
            if ((parent instanceof JScrollPane)) {
                JScrollPane scrollPane = (JScrollPane)parent;
                JViewport viewport = scrollPane.getViewport();
                if (viewport == null || SwingUtilities.getUnwrappedView(viewport) != this) {
                    return;
                }
                scrollPane.setColumnHeaderView(null);
                if ((scrollPane.getCorner("UPPER_TRAILING_CORNER") instanceof UIResource)) {
                    scrollPane.setCorner("UPPER_TRAILING_CORNER", null);
                }
            }
        }
    }
    
    @Override
    void setUIProperty(String s, Object o) {
        if (s == "rowHeight") {
            if (!this.isRowHeightSet) {
                this.setRowHeight(((Number)o).intValue());
                this.isRowHeightSet = false;
            }
            return;
        }
        else {
            super.setUIProperty(s, o);
            return;
        }
    }
    
    @Deprecated
    public static JScrollPane createScrollPaneForTable(JTable table) {
        return new JScrollPane(table);
    }
    
    public void setTableHeader(JTableHeader tableHeader) {
        if (this.tableHeader != tableHeader) {
            JTableHeader tableHeader2 = this.tableHeader;
            if (tableHeader2 != null) {
                tableHeader2.setTable(null);
            }
            if ((this.tableHeader = tableHeader) != null) {
                tableHeader.setTable(this);
            }
            this.firePropertyChange("tableHeader", tableHeader2, tableHeader);
        }
    }
    
    public JTableHeader getTableHeader() {
        return this.tableHeader;
    }
    
    public void setRowHeight(int rowHeight) {
        if (rowHeight <= 0) {
            throw new IllegalArgumentException("New row height less than 1");
        }
        int rowHeight2 = this.rowHeight;
        this.rowHeight = rowHeight;
        this.rowModel = null;
        if (this.sortManager != null) {
            this.sortManager.modelRowSizes = null;
        }
        this.isRowHeightSet = true;
        this.resizeAndRepaint();
        this.firePropertyChange("rowHeight", rowHeight2, rowHeight);
        return;
    }
    
    public int getRowHeight() {
        return this.rowHeight;
    }
    
    private SizeSequence getRowModel() {
        if (this.rowModel == null) {
            this.rowModel = new SizeSequence(this.getRowCount(), this.getRowHeight());
        }
        return this.rowModel;
    }
    
    public void setRowHeight(int n, int n2) {
        if (n2 <= 0) {
            throw new IllegalArgumentException("New row height less than 1");
        }
        this.getRowModel().setSize(n, n2);
        if (this.sortManager != null) {
            this.sortManager.setViewRowHeight(n, n2);
        }
        this.resizeAndRepaint();
        return;
    }
    
    public int getRowHeight(int n) {
        return (this.rowModel == null) ? this.getRowHeight() : this.rowModel.getSize(n);
    }
    
    public void setRowMargin(int rowMargin) {
        int rowMargin2 = this.rowMargin;
        this.rowMargin = rowMargin;
        this.resizeAndRepaint();
        this.firePropertyChange("rowMargin", rowMargin2, rowMargin);
    }
    
    public int getRowMargin() {
        return this.rowMargin;
    }
    
    public void setIntercellSpacing(Dimension dimension) {
        this.setRowMargin(dimension.height);
        this.getColumnModel().setColumnMargin(dimension.width);
        this.resizeAndRepaint();
    }
    
    public Dimension getIntercellSpacing() {
        return new Dimension(this.getColumnModel().getColumnMargin(), this.rowMargin);
    }
    
    public void setGridColor(Color gridColor) {
        if (gridColor == null) {
            throw new IllegalArgumentException("New color is null");
        }
        this.firePropertyChange("gridColor", this.gridColor, this.gridColor = gridColor);
        this.repaint();
        return;
    }
    
    public Color getGridColor() {
        return this.gridColor;
    }
    
    public void setShowGrid(boolean b) {
        this.setShowHorizontalLines(b);
        this.setShowVerticalLines(b);
        this.repaint();
    }
    
    public void setShowHorizontalLines(boolean showHorizontalLines) {
        this.firePropertyChange("showHorizontalLines", this.showHorizontalLines, this.showHorizontalLines = showHorizontalLines);
        this.repaint();
    }
    
    public void setShowVerticalLines(boolean showVerticalLines) {
        this.firePropertyChange("showVerticalLines", this.showVerticalLines, this.showVerticalLines = showVerticalLines);
        this.repaint();
    }
    
    public boolean getShowHorizontalLines() {
        return this.showHorizontalLines;
    }
    
    public boolean getShowVerticalLines() {
        return this.showVerticalLines;
    }
    
    public void setAutoResizeMode(int autoResizeMode) {
        if (autoResizeMode == 0 || autoResizeMode == 1 || autoResizeMode == 2 || autoResizeMode == 3 || autoResizeMode == 4) {
            int autoResizeMode2 = this.autoResizeMode;
            this.autoResizeMode = autoResizeMode;
            this.resizeAndRepaint();
            if (this.tableHeader != null) {
                this.tableHeader.resizeAndRepaint();
            }
            this.firePropertyChange("autoResizeMode", autoResizeMode2, this.autoResizeMode);
        }
    }
    
    public int getAutoResizeMode() {
        return this.autoResizeMode;
    }
    
    public void setAutoCreateColumnsFromModel(boolean autoCreateColumnsFromModel) {
        if (this.autoCreateColumnsFromModel != autoCreateColumnsFromModel) {
            int autoCreateColumnsFromModel2 = this.autoCreateColumnsFromModel;
            this.autoCreateColumnsFromModel = autoCreateColumnsFromModel;
            if (autoCreateColumnsFromModel) {
                this.createDefaultColumnsFromModel();
            }
            this.firePropertyChange("autoCreateColumnsFromModel", (boolean)autoCreateColumnsFromModel2, autoCreateColumnsFromModel);
        }
    }
    
    public boolean getAutoCreateColumnsFromModel() {
        return this.autoCreateColumnsFromModel;
    }
    
    public void createDefaultColumnsFromModel() {
        TableModel model = this.getModel();
        if (model != null) {
            TableColumnModel columnModel = this.getColumnModel();
            while (columnModel.getColumnCount() > 0) {
                columnModel.removeColumn(columnModel.getColumn(0));
            }
            for (int i = 0; i < model.getColumnCount(); ++i) {
                this.addColumn(new TableColumn(i));
            }
        }
    }
    
    public void setDefaultRenderer(Class<?> clazz, TableCellRenderer value) {
        if (value != null) {
            this.defaultRenderersByColumnClass.put(clazz, value);
        }
        else {
            this.defaultRenderersByColumnClass.remove(clazz);
        }
    }
    
    public TableCellRenderer getDefaultRenderer(Class<?> key) {
        if (key == null) {
            return null;
        }
        Object value = this.defaultRenderersByColumnClass.get(key);
        if (value != null) {
            return (TableCellRenderer)value;
        }
        Class<Object> superclass = key.getSuperclass();
        if (superclass == null && key != Object.class) {
            superclass = Object.class;
        }
        return this.getDefaultRenderer(superclass);
    }
    
    public void setDefaultEditor(Class<?> clazz, TableCellEditor value) {
        if (value != null) {
            this.defaultEditorsByColumnClass.put(clazz, value);
        }
        else {
            this.defaultEditorsByColumnClass.remove(clazz);
        }
    }
    
    public TableCellEditor getDefaultEditor(Class<?> key) {
        if (key == null) {
            return null;
        }
        Object value = this.defaultEditorsByColumnClass.get(key);
        if (value != null) {
            return (TableCellEditor)value;
        }
        return this.getDefaultEditor(key.getSuperclass());
    }
    
    public void setDragEnabled(boolean dragEnabled) {
        if (dragEnabled && GraphicsEnvironment.isHeadless()) {
            throw new HeadlessException();
        }
        this.dragEnabled = dragEnabled;
        return;
    }
    
    public boolean getDragEnabled() {
        return this.dragEnabled;
    }
    
    public final void setDropMode(DropMode dropMode) {
        if (dropMode != null) {
            switch (dropMode) {
                case USE_SELECTION:
                case ON:
                case INSERT:
                case INSERT_ROWS:
                case INSERT_COLS:
                case ON_OR_INSERT:
                case ON_OR_INSERT_ROWS:
                case ON_OR_INSERT_COLS: {
                    this.dropMode = dropMode;
                    return;
                }
                default: {
                    break;
                }
            }
        }
        throw new IllegalArgumentException(dropMode + ": Unsupported drop mode for table");
    }
    
    public final DropMode getDropMode() {
        return this.dropMode;
    }
    
    @Override
    JTable.DropLocation dropLocationForPoint(Point point) {
        Object o = null;
        int rowAtPoint = this.rowAtPoint(point);
        int columnAtPoint = this.columnAtPoint(point);
        int n = Boolean.TRUE == this.getClientProperty("Table.isFileList") && SwingUtilities2.pointOutsidePrefSize(this, rowAtPoint, columnAtPoint, point);
        Rectangle cellRect = this.getCellRect(rowAtPoint, columnAtPoint, true);
        boolean n2 = false;
        int leftToRight = this.getComponentOrientation().isLeftToRight();
        switch (this.dropMode) {
            case USE_SELECTION:
            case ON: {
                if (rowAtPoint == -1 || columnAtPoint == -1 || n) {
                    o = new JTable.DropLocation(point, -1, -1, false, false);
                    break;
                }
                o = new JTable.DropLocation(point, rowAtPoint, columnAtPoint, false, false);
                break;
            }
            case INSERT: {
                if (rowAtPoint == -1 && columnAtPoint == -1) {
                    o = new JTable.DropLocation(point, 0, 0, true, true);
                    break;
                }
                SwingUtilities2.Section liesInHorizontal = SwingUtilities2.liesInHorizontal(cellRect, point, (boolean)leftToRight, true);
                if (rowAtPoint == -1) {
                    if (liesInHorizontal == SwingUtilities2.Section.LEADING) {
                        o = new JTable.DropLocation(point, this.getRowCount(), columnAtPoint, true, true);
                        break;
                    }
                    if (liesInHorizontal == SwingUtilities2.Section.TRAILING) {
                        o = new JTable.DropLocation(point, this.getRowCount(), columnAtPoint + 1, true, true);
                        break;
                    }
                    o = new JTable.DropLocation(point, this.getRowCount(), columnAtPoint, true, false);
                    break;
                }
                else if (liesInHorizontal == SwingUtilities2.Section.LEADING || liesInHorizontal == SwingUtilities2.Section.TRAILING) {
                    SwingUtilities2.Section liesInVertical = SwingUtilities2.liesInVertical(cellRect, point, true);
                    if (liesInVertical == SwingUtilities2.Section.LEADING) {
                        n2 = true;
                    }
                    else if (liesInVertical == SwingUtilities2.Section.TRAILING) {
                        ++rowAtPoint;
                        n2 = true;
                    }
                    o = new JTable.DropLocation(point, rowAtPoint, (liesInHorizontal == SwingUtilities2.Section.TRAILING) ? (columnAtPoint + 1) : columnAtPoint, (boolean)n2, true);
                    break;
                }
                else {
                    if (SwingUtilities2.liesInVertical(cellRect, point, false) == SwingUtilities2.Section.TRAILING) {
                        ++rowAtPoint;
                    }
                    o = new JTable.DropLocation(point, rowAtPoint, columnAtPoint, true, false);
                    break;
                }
            }
            case INSERT_ROWS: {
                if (rowAtPoint == -1 && columnAtPoint == -1) {
                    o = new JTable.DropLocation(point, -1, -1, false, false);
                    break;
                }
                if (rowAtPoint == -1) {
                    o = new JTable.DropLocation(point, this.getRowCount(), columnAtPoint, true, false);
                    break;
                }
                if (SwingUtilities2.liesInVertical(cellRect, point, false) == SwingUtilities2.Section.TRAILING) {
                    ++rowAtPoint;
                }
                o = new JTable.DropLocation(point, rowAtPoint, columnAtPoint, true, false);
                break;
            }
            case ON_OR_INSERT_ROWS: {
                if (rowAtPoint == -1 && columnAtPoint == -1) {
                    o = new JTable.DropLocation(point, -1, -1, false, false);
                    break;
                }
                if (rowAtPoint == -1) {
                    o = new JTable.DropLocation(point, this.getRowCount(), columnAtPoint, true, false);
                    break;
                }
                SwingUtilities2.Section liesInVertical2 = SwingUtilities2.liesInVertical(cellRect, point, true);
                if (liesInVertical2 == SwingUtilities2.Section.LEADING) {
                    n2 = true;
                }
                else if (liesInVertical2 == SwingUtilities2.Section.TRAILING) {
                    ++rowAtPoint;
                    n2 = true;
                }
                o = new JTable.DropLocation(point, rowAtPoint, columnAtPoint, (boolean)n2, false);
                break;
            }
            case INSERT_COLS: {
                if (rowAtPoint == -1) {
                    o = new JTable.DropLocation(point, -1, -1, false, false);
                    break;
                }
                if (columnAtPoint == -1) {
                    o = new JTable.DropLocation(point, this.getColumnCount(), columnAtPoint, false, true);
                    break;
                }
                if (SwingUtilities2.liesInHorizontal(cellRect, point, (boolean)leftToRight, false) == SwingUtilities2.Section.TRAILING) {
                    ++columnAtPoint;
                }
                o = new JTable.DropLocation(point, rowAtPoint, columnAtPoint, false, true);
                break;
            }
            case ON_OR_INSERT_COLS: {
                if (rowAtPoint == -1) {
                    o = new JTable.DropLocation(point, -1, -1, false, false);
                    break;
                }
                if (columnAtPoint == -1) {
                    o = new JTable.DropLocation(point, rowAtPoint, this.getColumnCount(), false, true);
                    break;
                }
                SwingUtilities2.Section liesInHorizontal2 = SwingUtilities2.liesInHorizontal(cellRect, point, (boolean)leftToRight, true);
                if (liesInHorizontal2 == SwingUtilities2.Section.LEADING) {
                    n2 = true;
                }
                else if (liesInHorizontal2 == SwingUtilities2.Section.TRAILING) {
                    ++columnAtPoint;
                    n2 = true;
                }
                o = new JTable.DropLocation(point, rowAtPoint, columnAtPoint, false, (boolean)n2);
                break;
            }
            case ON_OR_INSERT: {
                if (rowAtPoint == -1 && columnAtPoint == -1) {
                    o = new JTable.DropLocation(point, 0, 0, true, true);
                    break;
                }
                SwingUtilities2.Section liesInHorizontal3 = SwingUtilities2.liesInHorizontal(cellRect, point, (boolean)leftToRight, true);
                if (rowAtPoint == -1) {
                    if (liesInHorizontal3 == SwingUtilities2.Section.LEADING) {
                        o = new JTable.DropLocation(point, this.getRowCount(), columnAtPoint, true, true);
                        break;
                    }
                    if (liesInHorizontal3 == SwingUtilities2.Section.TRAILING) {
                        o = new JTable.DropLocation(point, this.getRowCount(), columnAtPoint + 1, true, true);
                        break;
                    }
                    o = new JTable.DropLocation(point, this.getRowCount(), columnAtPoint, true, false);
                    break;
                }
                else {
                    SwingUtilities2.Section liesInVertical3 = SwingUtilities2.liesInVertical(cellRect, point, true);
                    if (liesInVertical3 == SwingUtilities2.Section.LEADING) {
                        n2 = true;
                    }
                    else if (liesInVertical3 == SwingUtilities2.Section.TRAILING) {
                        ++rowAtPoint;
                        n2 = true;
                    }
                    o = new JTable.DropLocation(point, rowAtPoint, (liesInHorizontal3 == SwingUtilities2.Section.TRAILING) ? (columnAtPoint + 1) : columnAtPoint, (boolean)n2, liesInHorizontal3 != SwingUtilities2.Section.MIDDLE);
                    break;
                }
            }
            default: {
                assert false : "Unexpected drop mode";
                break;
            }
        }
        return o;
    }
    
    @Override
    Object setDropLocation(TransferHandler.DropLocation dropLocation, Object o, boolean b) {
        Object o2 = null;
        JTable.DropLocation dropLocation2 = (JTable.DropLocation)dropLocation;
        if (this.dropMode == DropMode.USE_SELECTION) {
            if (dropLocation2 == null) {
                if (!b && o != null) {
                    this.clearSelection();
                    int[] array = ((int[][])(int[][])o)[0];
                    int[] array2 = ((int[][])(int[][])o)[1];
                    int[] array3 = ((int[][])(int[][])o)[2];
                    for (int n : array) {
                        this.addRowSelectionInterval(n, n);
                    }
                    for (int n2 : array2) {
                        this.addColumnSelectionInterval(n2, n2);
                    }
                    SwingUtilities2.setLeadAnchorWithoutSelection(this.getSelectionModel(), array3[1], array3[0]);
                    SwingUtilities2.setLeadAnchorWithoutSelection(this.getColumnModel().getSelectionModel(), array3[3], array3[2]);
                }
            }
            else {
                if (this.dropLocation == null) {
                    o2 = new int[][] { this.getSelectedRows(), this.getSelectedColumns(), { this.getAdjustedIndex(this.getSelectionModel().getAnchorSelectionIndex(), true), this.getAdjustedIndex(this.getSelectionModel().getLeadSelectionIndex(), true), this.getAdjustedIndex(this.getColumnModel().getSelectionModel().getAnchorSelectionIndex(), false), this.getAdjustedIndex(this.getColumnModel().getSelectionModel().getLeadSelectionIndex(), false) } };
                }
                else {
                    o2 = o;
                }
                if (dropLocation2.getRow() == -1) {
                    this.clearSelectionAndLeadAnchor();
                }
                else {
                    this.setRowSelectionInterval(dropLocation2.getRow(), dropLocation2.getRow());
                    this.setColumnSelectionInterval(dropLocation2.getColumn(), dropLocation2.getColumn());
                }
            }
        }
        this.firePropertyChange("dropLocation", this.dropLocation, this.dropLocation = dropLocation2);
        return o2;
    }
    
    public final JTable.DropLocation getDropLocation() {
        return this.dropLocation;
    }
    
    public void setAutoCreateRowSorter(boolean autoCreateRowSorter) {
        int autoCreateRowSorter2 = this.autoCreateRowSorter;
        this.autoCreateRowSorter = autoCreateRowSorter;
        if (autoCreateRowSorter) {
            this.setRowSorter(new TableRowSorter(this.getModel()));
        }
        this.firePropertyChange("autoCreateRowSorter", (boolean)autoCreateRowSorter2, autoCreateRowSorter);
    }
    
    public boolean getAutoCreateRowSorter() {
        return this.autoCreateRowSorter;
    }
    
    public void setUpdateSelectionOnSort(boolean updateSelectionOnSort) {
        if (this.updateSelectionOnSort != updateSelectionOnSort) {
            this.updateSelectionOnSort = updateSelectionOnSort;
            this.firePropertyChange("updateSelectionOnSort", !updateSelectionOnSort, updateSelectionOnSort);
        }
    }
    
    public boolean getUpdateSelectionOnSort() {
        return this.updateSelectionOnSort;
    }
    
    public void setRowSorter(RowSorter<? extends TableModel> rowSorter) {
        RowSorter<Object> sorter = null;
        if (this.sortManager != null) {
            sorter = this.sortManager.sorter;
            this.sortManager.dispose();
            this.sortManager = null;
        }
        this.rowModel = null;
        this.clearSelectionAndLeadAnchor();
        if (rowSorter != null) {
            this.sortManager = new JTable.SortManager(rowSorter);
        }
        this.resizeAndRepaint();
        this.firePropertyChange("rowSorter", sorter, rowSorter);
        this.firePropertyChange("sorter", sorter, rowSorter);
    }
    
    public RowSorter<? extends TableModel> getRowSorter() {
        return (this.sortManager != null) ? this.sortManager.sorter : null;
    }
    
    public void setSelectionMode(int n) {
        this.clearSelection();
        this.getSelectionModel().setSelectionMode(n);
        this.getColumnModel().getSelectionModel().setSelectionMode(n);
    }
    
    public void setRowSelectionAllowed(boolean rowSelectionAllowed) {
        int rowSelectionAllowed2 = this.rowSelectionAllowed;
        this.rowSelectionAllowed = rowSelectionAllowed;
        if (rowSelectionAllowed2 != (rowSelectionAllowed ? 1 : 0)) {
            this.repaint();
        }
        this.firePropertyChange("rowSelectionAllowed", (boolean)rowSelectionAllowed2, rowSelectionAllowed);
    }
    
    public boolean getRowSelectionAllowed() {
        return this.rowSelectionAllowed;
    }
    
    public void setColumnSelectionAllowed(boolean columnSelectionAllowed) {
        int columnSelectionAllowed2 = this.columnModel.getColumnSelectionAllowed();
        this.columnModel.setColumnSelectionAllowed(columnSelectionAllowed);
        if (columnSelectionAllowed2 != (columnSelectionAllowed ? 1 : 0)) {
            this.repaint();
        }
        this.firePropertyChange("columnSelectionAllowed", (boolean)columnSelectionAllowed2, columnSelectionAllowed);
    }
    
    public boolean getColumnSelectionAllowed() {
        return this.columnModel.getColumnSelectionAllowed();
    }
    
    public void setCellSelectionEnabled(boolean cellSelectionEnabled) {
        this.setRowSelectionAllowed(cellSelectionEnabled);
        this.setColumnSelectionAllowed(cellSelectionEnabled);
        this.firePropertyChange("cellSelectionEnabled", this.cellSelectionEnabled, this.cellSelectionEnabled = cellSelectionEnabled);
    }
    
    public boolean getCellSelectionEnabled() {
        return this.getRowSelectionAllowed() && this.getColumnSelectionAllowed();
    }
    
    public void selectAll() {
        if (this.isEditing()) {
            this.removeEditor();
        }
        if (this.getRowCount() > 0 && this.getColumnCount() > 0) {
            ListSelectionModel selectionModel = this.selectionModel;
            selectionModel.setValueIsAdjusting(true);
            int adjustedIndex = this.getAdjustedIndex(selectionModel.getLeadSelectionIndex(), true);
            int adjustedIndex2 = this.getAdjustedIndex(selectionModel.getAnchorSelectionIndex(), true);
            this.setRowSelectionInterval(0, this.getRowCount() - 1);
            SwingUtilities2.setLeadAnchorWithoutSelection(selectionModel, adjustedIndex, adjustedIndex2);
            selectionModel.setValueIsAdjusting(false);
            ListSelectionModel selectionModel2 = this.columnModel.getSelectionModel();
            selectionModel2.setValueIsAdjusting(true);
            int adjustedIndex3 = this.getAdjustedIndex(selectionModel2.getLeadSelectionIndex(), false);
            int adjustedIndex4 = this.getAdjustedIndex(selectionModel2.getAnchorSelectionIndex(), false);
            this.setColumnSelectionInterval(0, this.getColumnCount() - 1);
            SwingUtilities2.setLeadAnchorWithoutSelection(selectionModel2, adjustedIndex3, adjustedIndex4);
            selectionModel2.setValueIsAdjusting(false);
        }
    }
    
    public void clearSelection() {
        this.selectionModel.clearSelection();
        this.columnModel.getSelectionModel().clearSelection();
    }
    
    private void clearSelectionAndLeadAnchor() {
        this.selectionModel.setValueIsAdjusting(true);
        this.columnModel.getSelectionModel().setValueIsAdjusting(true);
        this.clearSelection();
        this.selectionModel.setAnchorSelectionIndex(-1);
        this.selectionModel.setLeadSelectionIndex(-1);
        this.columnModel.getSelectionModel().setAnchorSelectionIndex(-1);
        this.columnModel.getSelectionModel().setLeadSelectionIndex(-1);
        this.selectionModel.setValueIsAdjusting(false);
        this.columnModel.getSelectionModel().setValueIsAdjusting(false);
    }
    
    private int getAdjustedIndex(int n, boolean b) {
        return (n < (b ? this.getRowCount() : this.getColumnCount())) ? n : -1;
    }
    
    private int boundRow(int n) throws IllegalArgumentException {
        if (n < 0 || n >= this.getRowCount()) {
            throw new IllegalArgumentException("Row index out of range");
        }
        return n;
    }
    
    private int boundColumn(int n) {
        if (n < 0 || n >= this.getColumnCount()) {
            throw new IllegalArgumentException("Column index out of range");
        }
        return n;
    }
    
    public void setRowSelectionInterval(int n, int n2) {
        this.selectionModel.setSelectionInterval(this.boundRow(n), this.boundRow(n2));
    }
    
    public void setColumnSelectionInterval(int n, int n2) {
        this.columnModel.getSelectionModel().setSelectionInterval(this.boundColumn(n), this.boundColumn(n2));
    }
    
    public void addRowSelectionInterval(int n, int n2) {
        this.selectionModel.addSelectionInterval(this.boundRow(n), this.boundRow(n2));
    }
    
    public void addColumnSelectionInterval(int n, int n2) {
        this.columnModel.getSelectionModel().addSelectionInterval(this.boundColumn(n), this.boundColumn(n2));
    }
    
    public void removeRowSelectionInterval(int n, int n2) {
        this.selectionModel.removeSelectionInterval(this.boundRow(n), this.boundRow(n2));
    }
    
    public void removeColumnSelectionInterval(int n, int n2) {
        this.columnModel.getSelectionModel().removeSelectionInterval(this.boundColumn(n), this.boundColumn(n2));
    }
    
    public int getSelectedRow() {
        return this.selectionModel.getMinSelectionIndex();
    }
    
    public int getSelectedColumn() {
        return this.columnModel.getSelectionModel().getMinSelectionIndex();
    }
    
    public int[] getSelectedRows() {
        int i = this.selectionModel.getMinSelectionIndex();
        int maxSelectionIndex = this.selectionModel.getMaxSelectionIndex();
        if (i == -1 || maxSelectionIndex == -1) {
            return new int[0];
        }
        int[] array = new int[1 + (maxSelectionIndex - i)];
        int n = 0;
        for (; i <= maxSelectionIndex; ++i) {
            if (this.selectionModel.isSelectedIndex(i)) {
                array[n++] = i;
            }
        }
        int[] array2 = new int[n];
        System.arraycopy(array, 0, array2, 0, n);
        return array2;
    }
    
    public int[] getSelectedColumns() {
        return this.columnModel.getSelectedColumns();
    }
    
    public int getSelectedRowCount() {
        int i = this.selectionModel.getMinSelectionIndex();
        int maxSelectionIndex = this.selectionModel.getMaxSelectionIndex();
        int n = 0;
        for (; i <= maxSelectionIndex; ++i) {
            if (this.selectionModel.isSelectedIndex(i)) {
                ++n;
            }
        }
        return n;
    }
    
    public int getSelectedColumnCount() {
        return this.columnModel.getSelectedColumnCount();
    }
    
    public boolean isRowSelected(int n) {
        return this.selectionModel.isSelectedIndex(n);
    }
    
    public boolean isColumnSelected(int n) {
        return this.columnModel.getSelectionModel().isSelectedIndex(n);
    }
    
    public boolean isCellSelected(int n, int n2) {
        return (this.getRowSelectionAllowed() || this.getColumnSelectionAllowed()) && (!this.getRowSelectionAllowed() || this.isRowSelected(n)) && (!this.getColumnSelectionAllowed() || this.isColumnSelected(n2));
    }
    
    private void changeSelectionModel(ListSelectionModel listSelectionModel, int n, boolean b, boolean b2, boolean b3, int anchorSelectionIndex, boolean b4) {
        if (b2) {
            if (b) {
                if (b4) {
                    listSelectionModel.addSelectionInterval(anchorSelectionIndex, n);
                }
                else {
                    listSelectionModel.removeSelectionInterval(anchorSelectionIndex, n);
                    if (Boolean.TRUE == this.getClientProperty("Table.isFileList")) {
                        listSelectionModel.addSelectionInterval(n, n);
                        listSelectionModel.setAnchorSelectionIndex(anchorSelectionIndex);
                    }
                }
            }
            else {
                listSelectionModel.setSelectionInterval(anchorSelectionIndex, n);
            }
        }
        else if (b) {
            if (b3) {
                listSelectionModel.removeSelectionInterval(n, n);
            }
            else {
                listSelectionModel.addSelectionInterval(n, n);
            }
        }
        else {
            listSelectionModel.setSelectionInterval(n, n);
        }
    }
    
    public void changeSelection(int n, int n2, boolean b, boolean b2) {
        ListSelectionModel selectionModel = this.getSelectionModel();
        ListSelectionModel selectionModel2 = this.getColumnModel().getSelectionModel();
        int adjustedIndex = this.getAdjustedIndex(selectionModel.getAnchorSelectionIndex(), true);
        int adjustedIndex2 = this.getAdjustedIndex(selectionModel2.getAnchorSelectionIndex(), false);
        boolean n3 = true;
        if (adjustedIndex == -1) {
            if (this.getRowCount() > 0) {
                adjustedIndex = 0;
            }
            n3 = false;
        }
        if (adjustedIndex2 == -1) {
            if (this.getColumnCount() > 0) {
                adjustedIndex2 = 0;
            }
            n3 = false;
        }
        int cellSelected = this.isCellSelected(n, n2);
        int n4 = n3 && this.isCellSelected(adjustedIndex, adjustedIndex2);
        this.changeSelectionModel(selectionModel2, n2, b, b2, (boolean)cellSelected, adjustedIndex2, (boolean)n4);
        this.changeSelectionModel(selectionModel, n, b, b2, (boolean)cellSelected, adjustedIndex, (boolean)n4);
        if (this.getAutoscrolls()) {
            Rectangle cellRect = this.getCellRect(n, n2, false);
            if (cellRect != null) {
                this.scrollRectToVisible(cellRect);
            }
        }
    }
    
    public Color getSelectionForeground() {
        return this.selectionForeground;
    }
    
    public void setSelectionForeground(Color selectionForeground) {
        this.firePropertyChange("selectionForeground", this.selectionForeground, this.selectionForeground = selectionForeground);
        this.repaint();
    }
    
    public Color getSelectionBackground() {
        return this.selectionBackground;
    }
    
    public void setSelectionBackground(Color selectionBackground) {
        this.firePropertyChange("selectionBackground", this.selectionBackground, this.selectionBackground = selectionBackground);
        this.repaint();
    }
    
    public TableColumn getColumn(Object o) {
        TableColumnModel columnModel = this.getColumnModel();
        return columnModel.getColumn(columnModel.getColumnIndex(o));
    }
    
    public int convertColumnIndexToModel(int viewColumnIndex) {
        return SwingUtilities2.convertColumnIndexToModel(this.getColumnModel(), viewColumnIndex);
    }
    
    public int convertColumnIndexToView(int modelColumnIndex) {
        return SwingUtilities2.convertColumnIndexToView(this.getColumnModel(), modelColumnIndex);
    }
    
    public int convertRowIndexToView(int n) {
        RowSorter<Object> rowSorter = this.getRowSorter();
        if (rowSorter != null) {
            return rowSorter.convertRowIndexToView(n);
        }
        return n;
    }
    
    public int convertRowIndexToModel(int n) {
        RowSorter<Object> rowSorter = this.getRowSorter();
        if (rowSorter != null) {
            return rowSorter.convertRowIndexToModel(n);
        }
        return n;
    }
    
    public int getRowCount() {
        RowSorter<Object> rowSorter = this.getRowSorter();
        if (rowSorter != null) {
            return rowSorter.getViewRowCount();
        }
        return this.getModel().getRowCount();
    }
    
    public int getColumnCount() {
        return this.getColumnModel().getColumnCount();
    }
    
    public String getColumnName(int n) {
        return this.getModel().getColumnName(this.convertColumnIndexToModel(n));
    }
    
    public Class<?> getColumnClass(int n) {
        return this.getModel().getColumnClass(this.convertColumnIndexToModel(n));
    }
    
    public Object getValueAt(int n, int n2) {
        return this.getModel().getValueAt(this.convertRowIndexToModel(n), this.convertColumnIndexToModel(n2));
    }
    
    public void setValueAt(Object o, int n, int n2) {
        this.getModel().setValueAt(o, this.convertRowIndexToModel(n), this.convertColumnIndexToModel(n2));
    }
    
    public boolean isCellEditable(int n, int n2) {
        return this.getModel().isCellEditable(this.convertRowIndexToModel(n), this.convertColumnIndexToModel(n2));
    }
    
    public void addColumn(TableColumn tableColumn) {
        if (tableColumn.getHeaderValue() == null) {
            tableColumn.setHeaderValue(this.getModel().getColumnName(tableColumn.getModelIndex()));
        }
        this.getColumnModel().addColumn(tableColumn);
    }
    
    public void removeColumn(TableColumn tableColumn) {
        this.getColumnModel().removeColumn(tableColumn);
    }
    
    public void moveColumn(int n, int n2) {
        this.getColumnModel().moveColumn(n, n2);
    }
    
    public int columnAtPoint(Point point) {
        int x = point.x;
        if (!this.getComponentOrientation().isLeftToRight()) {
            x = this.getWidth() - x - 1;
        }
        return this.getColumnModel().getColumnIndexAtX(x);
    }
    
    public int rowAtPoint(Point point) {
        int y = point.y;
        int n = (this.rowModel == null) ? (y / this.getRowHeight()) : this.rowModel.getIndex(y);
        if (n < 0) {
            return -1;
        }
        if (n >= this.getRowCount()) {
            return -1;
        }
        return n;
    }
    
    public Rectangle getCellRect(int n, int n2, boolean b) {
        Rectangle rectangle = new Rectangle();
        boolean n3 = true;
        if (n < 0) {
            n3 = false;
        }
        else if (n >= this.getRowCount()) {
            rectangle.y = this.getHeight();
            n3 = false;
        }
        else {
            rectangle.height = this.getRowHeight(n);
            rectangle.y = ((this.rowModel == null) ? (n * rectangle.height) : this.rowModel.getPosition(n));
        }
        if (n2 < 0) {
            if (!this.getComponentOrientation().isLeftToRight()) {
                rectangle.x = this.getWidth();
            }
            n3 = false;
        }
        else if (n2 >= this.getColumnCount()) {
            if (this.getComponentOrientation().isLeftToRight()) {
                rectangle.x = this.getWidth();
            }
            n3 = false;
        }
        else {
            TableColumnModel columnModel = this.getColumnModel();
            if (this.getComponentOrientation().isLeftToRight()) {
                for (int j = 0; j < n2; ++j) {
                    rectangle.x += columnModel.getColumn(j).getWidth();
                }
            }
            else {
                for (int i = columnModel.getColumnCount() - 1; i > n2; --i) {
                    rectangle.x += columnModel.getColumn(i).getWidth();
                }
            }
            rectangle.width = columnModel.getColumn(n2).getWidth();
        }
        if (n3 && !b) {
            int min = Math.min(this.getRowMargin(), rectangle.height);
            int min2 = Math.min(this.getColumnModel().getColumnMargin(), rectangle.width);
            rectangle.setBounds(rectangle.x + min2 / 2, rectangle.y + min / 2, rectangle.width - min2, rectangle.height - min);
            return rectangle;
        }
        return rectangle;
    }
    
    private int viewIndexForColumn(TableColumn tableColumn) {
        TableColumnModel columnModel = this.getColumnModel();
        int i = 0;
        while (i < columnModel.getColumnCount()) {
            if (columnModel.getColumn(i) == tableColumn) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    @Override
    public void doLayout() {
        TableColumn resizingColumn = this.getResizingColumn();
        if (resizingColumn == null) {
            this.setWidthsFromPreferredWidths(false);
        }
        else {
            this.accommodateDelta(this.viewIndexForColumn(resizingColumn), this.getWidth() - this.getColumnModel().getTotalColumnWidth());
            int n = this.getWidth() - this.getColumnModel().getTotalColumnWidth();
            if (n != 0) {
                resizingColumn.setWidth(resizingColumn.getWidth() + n);
            }
            this.setWidthsFromPreferredWidths(true);
        }
        super.doLayout();
    }
    
    private TableColumn getResizingColumn() {
        return (this.tableHeader == null) ? null : this.tableHeader.getResizingColumn();
    }
    
    @Deprecated
    public void sizeColumnsToFit(boolean b) {
        int autoResizeMode = this.autoResizeMode;
        this.setAutoResizeMode(b ? 3 : 4);
        this.sizeColumnsToFit(-1);
        this.setAutoResizeMode(autoResizeMode);
    }
    
    public void sizeColumnsToFit(int n) {
        if (n == -1) {
            this.setWidthsFromPreferredWidths(false);
        }
        else if (this.autoResizeMode == 0) {
            TableColumn column = this.getColumnModel().getColumn(n);
            column.setPreferredWidth(column.getWidth());
        }
        else {
            this.accommodateDelta(n, this.getWidth() - this.getColumnModel().getTotalColumnWidth());
            this.setWidthsFromPreferredWidths(true);
        }
    }
    
    private void setWidthsFromPreferredWidths(final boolean b) {
        int width = this.getWidth();
        int width2 = this.getPreferredSize().width;
        width2 = (b ? width2 : width);
        this.adjustSizes((long)width2, new JTable.Resizable3() {
            
            @Override
            public int getElementCount() {
                return cm.getColumnCount();
            }
            
            @Override
            public int getLowerBoundAt(int n) {
                return cm.getColumn(n).getMinWidth();
            }
            
            @Override
            public int getUpperBoundAt(int n) {
                return cm.getColumn(n).getMaxWidth();
            }
            
            @Override
            public int getMidPointAt(int n) {
                if (!b) {
                    return cm.getColumn(n).getPreferredWidth();
                }
                return cm.getColumn(n).getWidth();
            }
            
            @Override
            public void setSizeAt(int n, int n2) {
                if (!b) {
                    cm.getColumn(n2).setWidth(n);
                }
                else {
                    cm.getColumn(n2).setPreferredWidth(n);
                }
            }
        }, b);
    }
    
    private void accommodateDelta(int n, int n2) {
        int columnCount = this.getColumnCount();
        final int i;
        final int min;
        switch (this.autoResizeMode) {
            case 1: {
                i = n + 1;
                min = Math.min(i + 1, columnCount);
                break;
            }
            case 2: {
                i = n + 1;
                min = columnCount;
                break;
            }
            case 3: {
                i = columnCount - 1;
                min = i + 1;
                break;
            }
            case 4: {
                i = 0;
                min = columnCount;
                break;
            }
            default: {
                return;
            }
        }
        JTable.3 o = new JTable.Resizable3() {
            
            @Override
            public int getElementCount() {
                return min - i;
            }
            
            @Override
            public int getLowerBoundAt(int n) {
                return cm.getColumn(n + i).getMinWidth();
            }
            
            @Override
            public int getUpperBoundAt(int n) {
                return cm.getColumn(n + i).getMaxWidth();
            }
            
            @Override
            public int getMidPointAt(int n) {
                return cm.getColumn(n + i).getWidth();
            }
            
            @Override
            public void setSizeAt(int width, int n) {
                cm.getColumn(n + i).setWidth(width);
            }
        };
        int n3 = 0;
        for (; i < min; ++i) {
            n3 += this.columnModel.getColumn(i).getWidth();
        }
        this.adjustSizes((long)(n3 + n2), o, false);
    }
    
    private void adjustSizes(long n, final JTable.Resizable3 resizable3, boolean b) {
        int elementCount = resizable3.getElementCount();
        long n2 = 0L;
        for (int i = 0; i < elementCount; ++i) {
            n2 += (long)resizable3.getMidPointAt(i);
        }
        Object o;
        if (n < n2 == !b) {
            o = new JTable.Resizable2() {
                @Override
                public int getElementCount() {
                    return resizable3.getElementCount();
                }
                
                @Override
                public int getLowerBoundAt(int n) {
                    return resizable3.getLowerBoundAt(n);
                }
                
                @Override
                public int getUpperBoundAt(int n) {
                    return resizable3.getMidPointAt(n);
                }
                
                @Override
                public void setSizeAt(int n, int n2) {
                    resizable3.setSizeAt(n, n2);
                }
            };
        }
        else {
            o = new JTable.Resizable2() {
                @Override
                public int getElementCount() {
                    return resizable3.getElementCount();
                }
                
                @Override
                public int getLowerBoundAt(int n) {
                    return resizable3.getMidPointAt(n);
                }
                
                @Override
                public int getUpperBoundAt(int n) {
                    return resizable3.getUpperBoundAt(n);
                }
                
                @Override
                public void setSizeAt(int n, int n2) {
                    resizable3.setSizeAt(n, n2);
                }
            };
        }
        this.adjustSizes(n, o, !b);
    }
    
    private void adjustSizes(long min, JTable.Resizable2 resizable2, boolean b) {
        long a = 0L;
        long b2 = 0L;
        for (int i = 0; i < resizable2.getElementCount(); ++i) {
            a += (long)resizable2.getLowerBoundAt(i);
            b2 += (long)resizable2.getUpperBoundAt(i);
        }
        if (b) {
            min = Math.min(Math.max(a, min), b2);
        }
        int lowerBound;
        int upperBound;
        for (int j = 0; j < resizable2.getElementCount(); min -= (long)lowerBound, a -= (long)lowerBound, b2 -= (long)upperBound, ++j) {
            lowerBound = resizable2.getLowerBoundAt(j);
            upperBound = resizable2.getUpperBoundAt(j);
            if (a == b2) {
            }
            else {
                lowerBound = (int)Math.round((double)lowerBound + (double)(min - a) / (double)(b2 - a) * (double)(upperBound - lowerBound));
            }
            resizable2.setSizeAt(lowerBound, j);
        }
    }
    
    @Override
    public String getToolTipText(MouseEvent mouseEvent) {
        String s = null;
        Point point = mouseEvent.getPoint();
        int columnAtPoint = this.columnAtPoint(point);
        int rowAtPoint = this.rowAtPoint(point);
        if (columnAtPoint != -1 && rowAtPoint != -1) {
            Component prepareRenderer = this.prepareRenderer(this.getCellRenderer(rowAtPoint, columnAtPoint), rowAtPoint, columnAtPoint);
            if ((prepareRenderer instanceof JComponent)) {
                Rectangle cellRect = this.getCellRect(rowAtPoint, columnAtPoint, false);
                point.translate(-cellRect.x, -cellRect.y);
                MouseEvent mouseEvent2 = new MouseEvent(prepareRenderer, mouseEvent.getID(), mouseEvent.getWhen(), mouseEvent.getModifiers(), point.x, point.y, mouseEvent.getXOnScreen(), mouseEvent.getYOnScreen(), mouseEvent.getClickCount(), mouseEvent.isPopupTrigger(), 0);
                AWTAccessor.MouseEventAccessor mouseEventAccessor = AWTAccessor.getMouseEventAccessor();
                mouseEventAccessor.setCausedByTouchEvent(mouseEvent2, mouseEventAccessor.isCausedByTouchEvent(mouseEvent));
                s = ((JComponent)prepareRenderer).getToolTipText(mouseEvent2);
            }
        }
        if (s == null) {
            s = super.getToolTipText();
            return s;
        }
        return s;
    }
    
    public void setSurrendersFocusOnKeystroke(boolean surrendersFocusOnKeystroke) {
        this.surrendersFocusOnKeystroke = surrendersFocusOnKeystroke;
    }
    
    public boolean getSurrendersFocusOnKeystroke() {
        return this.surrendersFocusOnKeystroke;
    }
    
    public boolean editCellAt(int n, int n2) {
        return this.editCellAt(n, n2, null);
    }
    
    public boolean editCellAt(int editingRow, int editingColumn, EventObject eventObject) {
        if (this.cellEditor != null && !this.cellEditor.stopCellEditing()) {
            return false;
        }
        if (editingRow < 0 || editingRow >= this.getRowCount() || editingColumn < 0 || editingColumn >= this.getColumnCount()) {
            return false;
        }
        if (!this.isCellEditable(editingRow, editingColumn)) {
            return false;
        }
        if (this.editorRemover == null) {
            KeyboardFocusManager currentKeyboardFocusManager = KeyboardFocusManager.getCurrentKeyboardFocusManager();
            currentKeyboardFocusManager.addPropertyChangeListener("permanentFocusOwner", this.editorRemover = new JTable.CellEditorRemover(currentKeyboardFocusManager));
        }
        TableCellEditor cellEditor = this.getCellEditor(editingRow, editingColumn);
        if (cellEditor != null && cellEditor.isCellEditable(eventObject)) {
            this.editorComp = this.prepareEditor(cellEditor, editingRow, editingColumn);
            if (this.editorComp == null) {
                this.removeEditor();
                return false;
            }
            this.editorComp.setBounds(this.getCellRect(editingRow, editingColumn, false));
            this.add(this.editorComp);
            this.editorComp.validate();
            this.editorComp.repaint();
            this.setCellEditor(cellEditor);
            this.setEditingRow(editingRow);
            this.setEditingColumn(editingColumn);
            cellEditor.addCellEditorListener(this);
            return true;
        }
        else {
            return false;
        }
    }
    
    public boolean isEditing() {
        return this.cellEditor != null;
    }
    
    public Component getEditorComponent() {
        return this.editorComp;
    }
    
    public int getEditingColumn() {
        return this.editingColumn;
    }
    
    public int getEditingRow() {
        return this.editingRow;
    }
    
    public TableUI getUI() {
        return (TableUI)this.ui;
    }
    
    public void setUI(TableUI ui) {
        if (this.ui != ui) {
            super.setUI(ui);
            this.repaint();
        }
    }
    
    @Override
    public void updateUI() {
        TableColumnModel columnModel = this.getColumnModel();
        for (int i = 0; i < columnModel.getColumnCount(); ++i) {
            TableColumn column = columnModel.getColumn(i);
            SwingUtilities.updateRendererOrEditorUI(column.getCellRenderer());
            SwingUtilities.updateRendererOrEditorUI(column.getCellEditor());
            SwingUtilities.updateRendererOrEditorUI(column.getHeaderRenderer());
        }
        Enumeration<Object> elements = this.defaultRenderersByColumnClass.elements();
        while (elements.hasMoreElements()) {
            SwingUtilities.updateRendererOrEditorUI(elements.nextElement());
        }
        Enumeration<Object> elements2 = this.defaultEditorsByColumnClass.elements();
        while (elements2.hasMoreElements()) {
            SwingUtilities.updateRendererOrEditorUI(elements2.nextElement());
        }
        if (this.tableHeader != null && this.tableHeader.getParent() == null) {
            this.tableHeader.updateUI();
        }
        this.configureEnclosingScrollPaneUI();
        this.setUI((TableUI)UIManager.getUI(this));
    }
    
    @Override
    public String getUIClassID() {
        return "TableUI";
    }
    
    public void setModel(TableModel tableModel) {
        if (tableModel == null) {
            throw new IllegalArgumentException("Cannot set a null TableModel");
        }
        if (this.dataModel != tableModel) {
            TableModel dataModel = this.dataModel;
            if (dataModel != null) {
                dataModel.removeTableModelListener(this);
            }
            (this.dataModel = tableModel).addTableModelListener(this);
            this.tableChanged(new TableModelEvent(tableModel, -1));
            this.firePropertyChange("model", dataModel, tableModel);
            if (this.getAutoCreateRowSorter()) {
                this.setRowSorter(new TableRowSorter(tableModel));
            }
        }
        return;
    }
    
    public TableModel getModel() {
        return this.dataModel;
    }
    
    public void setColumnModel(TableColumnModel newValue) {
        if (newValue == null) {
            throw new IllegalArgumentException("Cannot set a null ColumnModel");
        }
        TableColumnModel columnModel = this.columnModel;
        if (newValue != columnModel) {
            if (columnModel != null) {
                columnModel.removeColumnModelListener(this);
            }
            (this.columnModel = newValue).addColumnModelListener(this);
            if (this.tableHeader != null) {
                this.tableHeader.setColumnModel(newValue);
            }
            this.firePropertyChange("columnModel", columnModel, newValue);
            this.resizeAndRepaint();
        }
        return;
    }
    
    public TableColumnModel getColumnModel() {
        return this.columnModel;
    }
    
    public void setSelectionModel(ListSelectionModel listSelectionModel) {
        if (listSelectionModel == null) {
            throw new IllegalArgumentException("Cannot set a null SelectionModel");
        }
        ListSelectionModel selectionModel = this.selectionModel;
        if (listSelectionModel != selectionModel) {
            if (selectionModel != null) {
                selectionModel.removeListSelectionListener(this);
            }
            (this.selectionModel = listSelectionModel).addListSelectionListener(this);
            this.firePropertyChange("selectionModel", selectionModel, listSelectionModel);
            this.repaint();
        }
        return;
    }
    
    public ListSelectionModel getSelectionModel() {
        return this.selectionModel;
    }
    
    @Override
    public void sorterChanged(RowSorterEvent rowSorterEvent) {
        if (rowSorterEvent.getType() == RowSorterEvent.Type.SORT_ORDER_CHANGED) {
            JTableHeader tableHeader = this.getTableHeader();
            if (tableHeader != null) {
                tableHeader.repaint();
            }
        }
        else if (rowSorterEvent.getType() == RowSorterEvent.Type.SORTED) {
            this.sorterChanged = true;
            if (!this.ignoreSortChange) {
                this.sortedTableChanged(rowSorterEvent, null);
            }
        }
    }
    
    private void sortedTableChanged(RowSorterEvent rowSorterEvent, TableModelEvent tableModelEvent) {
        int convertRowIndexToModel = -1;
        JTable.ModelChange modelChange = (tableModelEvent != null) ? new JTable.ModelChange(tableModelEvent) : null;
        if ((modelChange == null || !modelChange.allRowsChanged) && this.editingRow != -1) {
            convertRowIndexToModel = this.convertRowIndexToModel(rowSorterEvent, this.editingRow);
        }
        this.sortManager.prepareForChange(rowSorterEvent, modelChange);
        if (tableModelEvent != null) {
            if (modelChange.type == 0) {
                this.repaintSortedRows(modelChange);
            }
            this.notifySorter(modelChange);
            if (modelChange.type != 0) {
                this.sorterChanged = true;
            }
        }
        else {
            this.sorterChanged = true;
        }
        this.sortManager.processChange(rowSorterEvent, modelChange, this.sorterChanged);
        if (this.sorterChanged) {
            if (this.editingRow != -1) {
                this.restoreSortingEditingRow((convertRowIndexToModel == -1) ? -1 : this.convertRowIndexToView(convertRowIndexToModel, modelChange));
            }
            if (tableModelEvent == null || modelChange.type != 0) {
                this.resizeAndRepaint();
            }
        }
        if (modelChange != null && modelChange.allRowsChanged) {
            this.clearSelectionAndLeadAnchor();
            this.resizeAndRepaint();
        }
    }
    
    private void repaintSortedRows(JTable.ModelChange modelChange) {
        if (modelChange.startModelIndex > modelChange.endModelIndex || modelChange.startModelIndex + 10 < modelChange.endModelIndex) {
            this.repaint();
            return;
        }
        int column = modelChange.event.getColumn();
        int convertColumnIndexToView;
        if ((column = column) == -1) {
            convertColumnIndexToView = 0;
        }
        else {
            convertColumnIndexToView = this.convertColumnIndexToView(column);
            if (convertColumnIndexToView == -1) {
                return;
            }
        }
        int i = modelChange.startModelIndex;
        while (i <= modelChange.endModelIndex) {
            int convertRowIndexToView = this.convertRowIndexToView(i++);
            if (convertRowIndexToView != -1) {
                Rectangle cellRect = this.getCellRect(convertRowIndexToView, convertColumnIndexToView, false);
                int x = cellRect.x;
                int width = cellRect.width;
                if (column == -1) {
                    x = 0;
                    width = this.getWidth();
                }
                this.repaint(x, cellRect.y, width, cellRect.height);
            }
            else {
                continue;
            }
        }
        return;
    }
    
    private void restoreSortingSelection(int[] array, int convertRowIndexToView, JTable.ModelChange modelChange) {
        for (int i = array.length - 1; i >= 0; --i) {
            array[i] = this.convertRowIndexToView(array[i], modelChange);
        }
        convertRowIndexToView = this.convertRowIndexToView(convertRowIndexToView, modelChange);
        if (array.length == 0 || (array.length == 1 && array[0] == this.getSelectedRow())) {
            return;
        }
        this.selectionModel.setValueIsAdjusting(true);
        this.selectionModel.clearSelection();
        for (int j = array.length - 1; j >= 0; --j) {
            if (array[j] != -1) {
                this.selectionModel.addSelectionInterval(array[j], array[j]);
            }
        }
        SwingUtilities2.setLeadAnchorWithoutSelection(this.selectionModel, convertRowIndexToView, convertRowIndexToView);
        this.selectionModel.setValueIsAdjusting(false);
        return;
    }
    
    private void restoreSortingEditingRow(int editingRow) {
        if (editingRow == -1) {
            TableCellEditor cellEditor = this.getCellEditor();
            if (cellEditor != null) {
                cellEditor.cancelCellEditing();
                if (this.getCellEditor() != null) {
                    this.removeEditor();
                }
            }
        }
        else {
            this.editingRow = editingRow;
            this.repaint();
        }
    }
    
    private void notifySorter(JTable.ModelChange modelChange) {
        try {
            this.ignoreSortChange = true;
            this.sorterChanged = false;
            switch (modelChange.type) {
                case 0: {
                    if (modelChange.event.getLastRow() == Integer.MAX_VALUE) {
                        this.sortManager.sorter.allRowsChanged();
                        break;
                    }
                    if (modelChange.event.getColumn() == -1) {
                        this.sortManager.sorter.rowsUpdated(modelChange.startModelIndex, modelChange.endModelIndex);
                        break;
                    }
                    this.sortManager.sorter.rowsUpdated(modelChange.startModelIndex, modelChange.endModelIndex, modelChange.event.getColumn());
                    break;
                }
                case 1: {
                    this.sortManager.sorter.rowsInserted(modelChange.startModelIndex, modelChange.endModelIndex);
                    break;
                }
                case -1: {
                    this.sortManager.sorter.rowsDeleted(modelChange.startModelIndex, modelChange.endModelIndex);
                    break;
                }
            }
        }
        finally {
            this.ignoreSortChange = false;
        }
    }
    
    private int convertRowIndexToView(int n, JTable.ModelChange modelChange) {
        if (n < 0) {
            return -1;
        }
        if (modelChange != null && n >= modelChange.startModelIndex) {
            if (modelChange.type == 1) {
                if (n + modelChange.length >= modelChange.modelRowCount) {
                    return -1;
                }
                return this.sortManager.sorter.convertRowIndexToView(n + modelChange.length);
            }
            else if (modelChange.type == -1) {
                if (n <= modelChange.endModelIndex) {
                    return -1;
                }
                if (n - modelChange.length >= modelChange.modelRowCount) {
                    return -1;
                }
                return this.sortManager.sorter.convertRowIndexToView(n - modelChange.length);
            }
        }
        if (n >= this.getModel().getRowCount()) {
            return -1;
        }
        return this.sortManager.sorter.convertRowIndexToView(n);
    }
    
    private int[] convertSelectionToModel(RowSorterEvent rowSorterEvent) {
        int[] selectedRows = this.getSelectedRows();
        for (int i = selectedRows.length - 1; i >= 0; --i) {
            selectedRows[i] = this.convertRowIndexToModel(rowSorterEvent, selectedRows[i]);
        }
        return selectedRows;
    }
    
    private int convertRowIndexToModel(RowSorterEvent rowSorterEvent, int index) {
        if (rowSorterEvent != null) {
            if (rowSorterEvent.getPreviousRowCount() == 0) {
                return index;
            }
            return rowSorterEvent.convertPreviousRowIndexToModel(index);
        }
        else {
            if (index < 0 || index >= this.getRowCount()) {
                return -1;
            }
            return this.convertRowIndexToModel(index);
        }
    }
    
    @Override
    public void tableChanged(TableModelEvent tableModelEvent) {
        if (tableModelEvent == null || tableModelEvent.getFirstRow() == -1) {
            this.clearSelectionAndLeadAnchor();
            this.rowModel = null;
            if (this.sortManager != null) {
                try {
                    this.ignoreSortChange = true;
                    this.sortManager.sorter.modelStructureChanged();
                }
                finally {
                    this.ignoreSortChange = false;
                }
                this.sortManager.allChanged();
            }
            if (this.getAutoCreateColumnsFromModel()) {
                this.createDefaultColumnsFromModel();
                return;
            }
            this.resizeAndRepaint();
            return;
        }
        else {
            if (this.sortManager != null) {
                this.sortedTableChanged(null, tableModelEvent);
                return;
            }
            if (this.rowModel != null) {
                this.repaint();
            }
            if (tableModelEvent.getType() == 1) {
                this.tableRowsInserted(tableModelEvent);
                return;
            }
            if (tableModelEvent.getType() == -1) {
                this.tableRowsDeleted(tableModelEvent);
                return;
            }
            int column = tableModelEvent.getColumn();
            int firstRow = tableModelEvent.getFirstRow();
            int lastRow = tableModelEvent.getLastRow();
            Rectangle cellRect;
            if (column == -1) {
                cellRect = new Rectangle(0, firstRow * this.getRowHeight(), this.getColumnModel().getTotalColumnWidth(), 0);
            }
            else {
                cellRect = this.getCellRect(firstRow, this.convertColumnIndexToView(column), false);
            }
            if (lastRow != Integer.MAX_VALUE) {
                cellRect.height = (lastRow - firstRow + 1) * this.getRowHeight();
                this.repaint(cellRect.x, cellRect.y, cellRect.width, cellRect.height);
            }
            else {
                this.clearSelectionAndLeadAnchor();
                this.resizeAndRepaint();
                this.rowModel = null;
            }
            return;
        }
    }
    
    private void tableRowsInserted(TableModelEvent tableModelEvent) {
        int firstRow = tableModelEvent.getFirstRow();
        int lastRow = tableModelEvent.getLastRow();
        if (firstRow < 0) {
            firstRow = 0;
        }
        if (lastRow < 0) {
            lastRow = this.getRowCount() - 1;
        }
        int n = lastRow - firstRow + 1;
        this.selectionModel.insertIndexInterval(firstRow, n, true);
        if (this.rowModel != null) {
            this.rowModel.insertEntries(firstRow, n, this.getRowHeight());
        }
        int rowHeight = this.getRowHeight();
        Rectangle o = new Rectangle(0, firstRow * rowHeight, this.getColumnModel().getTotalColumnWidth(), (this.getRowCount() - firstRow) * rowHeight);
        this.revalidate();
        this.repaint(o);
    }
    
    private void tableRowsDeleted(TableModelEvent tableModelEvent) {
        int firstRow = tableModelEvent.getFirstRow();
        int lastRow = tableModelEvent.getLastRow();
        if (firstRow < 0) {
            firstRow = 0;
        }
        if (lastRow < 0) {
            lastRow = this.getRowCount() - 1;
        }
        int n = lastRow - firstRow + 1;
        int n2 = this.getRowCount() + n;
        this.selectionModel.removeIndexInterval(firstRow, lastRow);
        if (this.rowModel != null) {
            this.rowModel.removeEntries(firstRow, n);
        }
        int rowHeight = this.getRowHeight();
        Rectangle o = new Rectangle(0, firstRow * rowHeight, this.getColumnModel().getTotalColumnWidth(), (n2 - firstRow) * rowHeight);
        this.revalidate();
        this.repaint(o);
    }
    
    @Override
    public void columnAdded(TableColumnModelEvent tableColumnModelEvent) {
        if (this.isEditing()) {
            this.removeEditor();
        }
        this.resizeAndRepaint();
    }
    
    @Override
    public void columnRemoved(TableColumnModelEvent tableColumnModelEvent) {
        if (this.isEditing()) {
            this.removeEditor();
        }
        this.resizeAndRepaint();
    }
    
    @Override
    public void columnMoved(TableColumnModelEvent tableColumnModelEvent) {
        if (this.isEditing() && !this.getCellEditor().stopCellEditing()) {
            this.getCellEditor().cancelCellEditing();
        }
        this.repaint();
    }
    
    @Override
    public void columnMarginChanged(ChangeEvent changeEvent) {
        if (this.isEditing() && !this.getCellEditor().stopCellEditing()) {
            this.getCellEditor().cancelCellEditing();
        }
        TableColumn resizingColumn = this.getResizingColumn();
        if (resizingColumn != null && this.autoResizeMode == 0) {
            resizingColumn.setPreferredWidth(resizingColumn.getWidth());
        }
        this.resizeAndRepaint();
    }
    
    private int limit(int a, int b, int a2) {
        return Math.min(a2, Math.max(a, b));
    }
    
    @Override
    public void columnSelectionChanged(ListSelectionEvent listSelectionEvent) {
        int valueIsAdjusting = listSelectionEvent.getValueIsAdjusting();
        if (this.columnSelectionAdjusting && valueIsAdjusting == 0) {
            this.columnSelectionAdjusting = false;
            return;
        }
        this.columnSelectionAdjusting = valueIsAdjusting;
        if (this.getRowCount() <= 0 || this.getColumnCount() <= 0) {
            return;
        }
        int limit = this.limit(listSelectionEvent.getFirstIndex(), 0, this.getColumnCount() - 1);
        int limit2 = this.limit(listSelectionEvent.getLastIndex(), 0, this.getColumnCount() - 1);
        int a = 0;
        int a2 = this.getRowCount() - 1;
        if (this.getRowSelectionAllowed()) {
            a = this.selectionModel.getMinSelectionIndex();
            a2 = this.selectionModel.getMaxSelectionIndex();
            int adjustedIndex = this.getAdjustedIndex(this.selectionModel.getLeadSelectionIndex(), true);
            if (a == -1 || a2 == -1) {
                if (adjustedIndex == -1) {
                    return;
                }
                a2 = (a = adjustedIndex);
            }
            else if (adjustedIndex != -1) {
                a = Math.min(a, adjustedIndex);
                a2 = Math.max(a2, adjustedIndex);
            }
        }
        this.repaint(this.getCellRect(a, limit, false).union(this.getCellRect(a2, limit2, false)));
        return;
    }
    
    @Override
    public void valueChanged(ListSelectionEvent listSelectionEvent) {
        if (this.sortManager != null) {
            this.sortManager.viewSelectionChanged(listSelectionEvent);
        }
        int valueIsAdjusting = listSelectionEvent.getValueIsAdjusting();
        if (this.rowSelectionAdjusting && valueIsAdjusting == 0) {
            this.rowSelectionAdjusting = false;
            return;
        }
        this.rowSelectionAdjusting = valueIsAdjusting;
        if (this.getRowCount() <= 0 || this.getColumnCount() <= 0) {
            return;
        }
        this.repaint(this.getCellRect(this.limit(listSelectionEvent.getFirstIndex(), 0, this.getRowCount() - 1), 0, false).union(this.getCellRect(this.limit(listSelectionEvent.getLastIndex(), 0, this.getRowCount() - 1), this.getColumnCount() - 1, false)));
        return;
    }
    
    @Override
    public void editingStopped(ChangeEvent changeEvent) {
        TableCellEditor cellEditor = this.getCellEditor();
        if (cellEditor != null) {
            this.setValueAt(cellEditor.getCellEditorValue(), this.editingRow, this.editingColumn);
            this.removeEditor();
        }
    }
    
    @Override
    public void editingCanceled(ChangeEvent changeEvent) {
        this.removeEditor();
    }
    
    public void setPreferredScrollableViewportSize(Dimension preferredViewportSize) {
        this.preferredViewportSize = preferredViewportSize;
    }
    
    @Override
    public Dimension getPreferredScrollableViewportSize() {
        return this.preferredViewportSize;
    }
    
    @Override
    public int getScrollableUnitIncrement(Rectangle rectangle, int n, int n2) {
        int leadingRow = this.getLeadingRow(rectangle);
        int leadingCol = this.getLeadingCol(rectangle);
        if (n == 1 && leadingRow < 0) {
            return this.getRowHeight();
        }
        if (n == 0 && leadingCol < 0) {
            return 100;
        }
        Rectangle cellRect = this.getCellRect(leadingRow, leadingCol, true);
        int leadingEdge = this.leadingEdge(rectangle, n);
        int leadingEdge2 = this.leadingEdge(cellRect, n);
        int n3;
        if (n == 1) {
            n3 = cellRect.height;
        }
        else {
            n3 = cellRect.width;
        }
        if (leadingEdge == leadingEdge2) {
            if (n2 < 0) {
                int n5 = 0;
                if (n == 1) {
                    while (--leadingRow >= 0) {
                        n5 = this.getRowHeight(leadingRow);
                        if (n5 == 0) {
                            continue;
                        }
                        break;
                    }
                }
                else {
                    while (--leadingCol >= 0) {
                        n5 = this.getCellRect(leadingRow, leadingCol, true).width;
                        if (n5 != 0) {
                            break;
                        }
                        continue;
                    }
                }
                return n5;
            }
            else {
                return n3;
            }
        }
        else {
            int abs = Math.abs(leadingEdge - leadingEdge2);
            int n4 = n3 - abs;
            if (n2 > 0) {
                return n4;
            }
            return abs;
        }
    }
    
    @Override
    public int getScrollableBlockIncrement(Rectangle rectangle, int n, int n2) {
        if (this.getRowCount() == 0) {
            if (1 == n) {
                int rowHeight2 = this.getRowHeight();
                return (rowHeight2 > 0) ? Math.max(rowHeight2, rectangle.height / rowHeight2 * rowHeight2) : rectangle.height;
            }
            return rectangle.width;
        }
        else {
            if (null == this.rowModel && 1 == n) {
                int rowAtPoint = this.rowAtPoint(rectangle.getLocation());
                assert rowAtPoint != -1;
                if (this.getCellRect(rowAtPoint, this.columnAtPoint(rectangle.getLocation()), true).y == rectangle.y) {
                    int rowHeight = this.getRowHeight();
                    assert rowHeight > 0;
                    return Math.max(rowHeight, rectangle.height / rowHeight * rowHeight);
                }
            }
            if (n2 < 0) {
                return this.getPreviousBlockIncrement(rectangle, n);
            }
            return this.getNextBlockIncrement(rectangle, n);
        }
    }
    
    private int getPreviousBlockIncrement(Rectangle rectangle, int n) {
        int leadingEdge = this.leadingEdge(rectangle, n);
        int leftToRight = this.getComponentOrientation().isLeftToRight();
        int n2;
        Point point;
        if (n == 1) {
            n2 = leadingEdge - rectangle.height;
            point = new Point(rectangle.x + (leftToRight ? 0 : rectangle.width), n2);
        }
        else if (!(leftToRight == 0)) {
            n2 = leadingEdge - rectangle.width;
            point = new Point(n2, rectangle.y);
        }
        else {
            n2 = leadingEdge + rectangle.width;
            point = new Point(n2 - 1, rectangle.y);
        }
        int rowAtPoint = this.rowAtPoint(point);
        int columnAtPoint = this.columnAtPoint(point);
        int width;
        if ((n == 1 & rowAtPoint < 0)) {
            width = 0;
        }
        else if ((n == 0 & columnAtPoint < 0)) {
            if (!(leftToRight == 0)) {
                width = 0;
            }
            else {
                width = this.getWidth();
            }
        }
        else {
            Rectangle cellRect = this.getCellRect(rowAtPoint, columnAtPoint, true);
            int leadingEdge2 = this.leadingEdge(cellRect, n);
            int trailingEdge = this.trailingEdge(cellRect, n);
            if ((n == 1 || leftToRight) && trailingEdge >= leadingEdge) {
                width = leadingEdge2;
            }
            else if (n == 0 && leftToRight == 0 && trailingEdge <= leadingEdge) {
                width = leadingEdge2;
            }
            else if (n2 == leadingEdge2) {
                width = leadingEdge2;
            }
            else {
                width = trailingEdge;
            }
        }
        return Math.abs(leadingEdge - width);
    }
    
    private int getNextBlockIncrement(Rectangle rectangle, int n) {
        int trailingRow = this.getTrailingRow(rectangle);
        int trailingCol = this.getTrailingCol(rectangle);
        int leadingEdge = this.leadingEdge(rectangle, n);
        if (n == 1 && trailingRow < 0) {
            return rectangle.height;
        }
        if (n == 0 && trailingCol < 0) {
            return rectangle.width;
        }
        Rectangle cellRect = this.getCellRect(trailingRow, trailingCol, true);
        int leadingEdge2 = this.leadingEdge(cellRect, n);
        int trailingEdge = this.trailingEdge(cellRect, n);
        int n2;
        if (n == 1 || this.getComponentOrientation().isLeftToRight()) {
            n2 = (leadingEdge2 <= leadingEdge);
        }
        else {
            n2 = (leadingEdge2 >= leadingEdge);
        }
        if (n2 != 0) {
        }
        else if (trailingEdge == this.trailingEdge(rectangle, n)) {
        }
        else {
            trailingEdge = leadingEdge2;
        }
        return Math.abs(trailingEdge - leadingEdge);
    }
    
    private int getLeadingRow(Rectangle rectangle) {
        Point point;
        if (this.getComponentOrientation().isLeftToRight()) {
            point = new Point(rectangle.x, rectangle.y);
        }
        else {
            point = new Point(rectangle.x + rectangle.width - 1, rectangle.y);
        }
        return this.rowAtPoint(point);
    }
    
    private int getLeadingCol(Rectangle rectangle) {
        Point point;
        if (this.getComponentOrientation().isLeftToRight()) {
            point = new Point(rectangle.x, rectangle.y);
        }
        else {
            point = new Point(rectangle.x + rectangle.width - 1, rectangle.y);
        }
        return this.columnAtPoint(point);
    }
    
    private int getTrailingRow(Rectangle rectangle) {
        Point point;
        if (this.getComponentOrientation().isLeftToRight()) {
            point = new Point(rectangle.x, rectangle.y + rectangle.height - 1);
        }
        else {
            point = new Point(rectangle.x + rectangle.width - 1, rectangle.y + rectangle.height - 1);
        }
        return this.rowAtPoint(point);
    }
    
    private int getTrailingCol(Rectangle rectangle) {
        Point point;
        if (this.getComponentOrientation().isLeftToRight()) {
            point = new Point(rectangle.x + rectangle.width - 1, rectangle.y);
        }
        else {
            point = new Point(rectangle.x, rectangle.y);
        }
        return this.columnAtPoint(point);
    }
    
    private int leadingEdge(Rectangle rectangle, int n) {
        if (n == 1) {
            return rectangle.y;
        }
        if (this.getComponentOrientation().isLeftToRight()) {
            return rectangle.x;
        }
        return rectangle.x + rectangle.width;
    }
    
    private int trailingEdge(Rectangle rectangle, int n) {
        if (n == 1) {
            return rectangle.y + rectangle.height;
        }
        if (this.getComponentOrientation().isLeftToRight()) {
            return rectangle.x + rectangle.width;
        }
        return rectangle.x;
    }
    
    @Override
    public boolean getScrollableTracksViewportWidth() {
        return this.autoResizeMode != 0;
    }
    
    @Override
    public boolean getScrollableTracksViewportHeight() {
        Container unwrappedParent = SwingUtilities.getUnwrappedParent(this);
        return this.getFillsViewportHeight() && unwrappedParent instanceof JViewport && unwrappedParent.getHeight() > this.getPreferredSize().height;
    }
    
    public void setFillsViewportHeight(boolean fillsViewportHeight) {
        int fillsViewportHeight2 = this.fillsViewportHeight;
        this.fillsViewportHeight = fillsViewportHeight;
        this.resizeAndRepaint();
        this.firePropertyChange("fillsViewportHeight", (boolean)fillsViewportHeight2, fillsViewportHeight);
    }
    
    public boolean getFillsViewportHeight() {
        return this.fillsViewportHeight;
    }
    
    @Override
    protected boolean processKeyBinding(KeyStroke keyStroke, KeyEvent keyEvent, int n, boolean b) {
        int n2 = super.processKeyBinding(keyStroke, keyEvent, n, b);
        if (n2 == 0 && n == 1 && this.isFocusOwner() && !Boolean.FALSE.equals(this.getClientProperty("JTable.autoStartsEdit"))) {
            Component component = this.getEditorComponent();
            if (component == null) {
                if (keyEvent == null || keyEvent.getID() != 401) {
                    return false;
                }
                int keyCode = keyEvent.getKeyCode();
                if (keyCode == 16 || keyCode == 17 || keyCode == 18) {
                    return false;
                }
                int leadSelectionIndex = this.getSelectionModel().getLeadSelectionIndex();
                int leadSelectionIndex2 = this.getColumnModel().getSelectionModel().getLeadSelectionIndex();
                if (leadSelectionIndex != -1 && leadSelectionIndex2 != -1 && !this.isEditing() && !this.editCellAt(leadSelectionIndex, leadSelectionIndex2, keyEvent)) {
                    return false;
                }
                component = this.getEditorComponent();
                if (component == null) {
                    return false;
                }
            }
            if ((component instanceof JComponent)) {
                n2 = ((JComponent)component).processKeyBinding(keyStroke, keyEvent, 0, b);
                if (this.getSurrendersFocusOnKeystroke()) {
                    component.requestFocus();
                }
            }
        }
        return n2;
    }
    
    protected void createDefaultRenderers() {
        (this.defaultRenderersByColumnClass = new UIDefaults(8, 0.75f)).put(Object.class, p0/* javax.swing.UIDefaults, */ -> new DefaultTableCellRenderer.UIResource());
        this.defaultRenderersByColumnClass.put(Number.class, p0/* javax.swing.UIDefaults, */ -> new JTable.NumberRenderer());
        this.defaultRenderersByColumnClass.put(Float.class, p0/* javax.swing.UIDefaults, */ -> new JTable.DoubleRenderer());
        this.defaultRenderersByColumnClass.put(Double.class, p0/* javax.swing.UIDefaults, */ -> new JTable.DoubleRenderer());
        this.defaultRenderersByColumnClass.put(Date.class, p0/* javax.swing.UIDefaults, */ -> new JTable.DateRenderer());
        this.defaultRenderersByColumnClass.put(Icon.class, p0/* javax.swing.UIDefaults, */ -> new JTable.IconRenderer());
        this.defaultRenderersByColumnClass.put(ImageIcon.class, p0/* javax.swing.UIDefaults, */ -> new JTable.IconRenderer());
        this.defaultRenderersByColumnClass.put(Boolean.class, p0/* javax.swing.UIDefaults, */ -> new JTable.BooleanRenderer());
    }
    
    protected void createDefaultEditors() {
        (this.defaultEditorsByColumnClass = new UIDefaults(3, 0.75f)).put(Object.class, p0/* javax.swing.UIDefaults, */ -> new JTable.GenericEditor());
        this.defaultEditorsByColumnClass.put(Number.class, p0/* javax.swing.UIDefaults, */ -> new JTable.NumberEditor());
        this.defaultEditorsByColumnClass.put(Boolean.class, p0/* javax.swing.UIDefaults, */ -> new JTable.BooleanEditor());
    }
    
    protected void initializeLocalVars() {
        this.setOpaque(this.updateSelectionOnSort = true);
        this.createDefaultRenderers();
        this.createDefaultEditors();
        this.setTableHeader(this.createDefaultTableHeader());
        this.setShowGrid(true);
        this.setAutoResizeMode(2);
        this.setRowHeight(16);
        this.isRowHeightSet = false;
        this.setRowMargin(1);
        this.setRowSelectionAllowed(true);
        this.setCellEditor(null);
        this.setEditingColumn(-1);
        this.setEditingRow(-1);
        this.setSurrendersFocusOnKeystroke(false);
        this.setPreferredScrollableViewportSize(new Dimension(450, 400));
        ToolTipManager.sharedInstance().registerComponent(this);
        this.setAutoscrolls(true);
    }
    
    protected TableModel createDefaultDataModel() {
        return new DefaultTableModel();
    }
    
    protected TableColumnModel createDefaultColumnModel() {
        return new DefaultTableColumnModel();
    }
    
    protected ListSelectionModel createDefaultSelectionModel() {
        return new DefaultListSelectionModel();
    }
    
    protected JTableHeader createDefaultTableHeader() {
        return new JTableHeader(this.columnModel);
    }
    
    protected void resizeAndRepaint() {
        this.revalidate();
        this.repaint();
    }
    
    public TableCellEditor getCellEditor() {
        return this.cellEditor;
    }
    
    public void setCellEditor(TableCellEditor cellEditor) {
        this.firePropertyChange("tableCellEditor", this.cellEditor, this.cellEditor = cellEditor);
    }
    
    public void setEditingColumn(int editingColumn) {
        this.editingColumn = editingColumn;
    }
    
    public void setEditingRow(int editingRow) {
        this.editingRow = editingRow;
    }
    
    public TableCellRenderer getCellRenderer(int n, int n2) {
        TableCellRenderer tableCellRenderer = this.getColumnModel().getColumn(n2).getCellRenderer();
        if (tableCellRenderer == null) {
            tableCellRenderer = this.getDefaultRenderer(this.getColumnClass(n2));
            return tableCellRenderer;
        }
        return tableCellRenderer;
    }
    
    public Component prepareRenderer(TableCellRenderer tableCellRenderer, int n, int n2) {
        Object value = this.getValueAt(n, n2);
        boolean cellSelected = false;
        boolean n3 = false;
        if (!this.isPaintingForPrint()) {
            cellSelected = this.isCellSelected(n, n2);
            int n4 = this.selectionModel.getLeadSelectionIndex() == n;
            int n5 = this.columnModel.getSelectionModel().getLeadSelectionIndex() == n2;
            n3 = (n4 && n5 && this.isFocusOwner());
        }
        return tableCellRenderer.getTableCellRendererComponent(this, value, (boolean)cellSelected, (boolean)n3, n, n2);
    }
    
    public TableCellEditor getCellEditor(int n, int n2) {
        TableCellEditor tableCellEditor = this.getColumnModel().getColumn(n2).getCellEditor();
        if (tableCellEditor == null) {
            tableCellEditor = this.getDefaultEditor(this.getColumnClass(n2));
            return tableCellEditor;
        }
        return tableCellEditor;
    }
    
    public Component prepareEditor(TableCellEditor tableCellEditor, int n, int n2) {
        Component tableCellEditorComponent = tableCellEditor.getTableCellEditorComponent(this, this.getValueAt(n, n2), this.isCellSelected(n, n2), n, n2);
        if ((tableCellEditorComponent instanceof JComponent)) {
            JComponent component = (JComponent)tableCellEditorComponent;
            if (component.getNextFocusableComponent() == null) {
                component.setNextFocusableComponent(this);
            }
        }
        return tableCellEditorComponent;
    }
    
    public void removeEditor() {
        KeyboardFocusManager.getCurrentKeyboardFocusManager().removePropertyChangeListener("permanentFocusOwner", this.editorRemover);
        this.editorRemover = null;
        TableCellEditor cellEditor = this.getCellEditor();
        if (cellEditor != null) {
            cellEditor.removeCellEditorListener(this);
            if (this.editorComp != null) {
                Component focusOwner = KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
                int n = focusOwner != null && SwingUtilities.isDescendingFrom(focusOwner, this);
                this.remove(this.editorComp);
                if (!(n == 0)) {
                    this.requestFocusInWindow();
                }
            }
            Rectangle cellRect = this.getCellRect(this.editingRow, this.editingColumn, false);
            this.setCellEditor(null);
            this.setEditingColumn(-1);
            this.setEditingRow(-1);
            this.editorComp = null;
            this.repaint(cellRect);
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("TableUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        if (this.ui != null && this.getUIClassID().equals("TableUI")) {
            this.ui.installUI(this);
        }
        this.createDefaultRenderers();
        this.createDefaultEditors();
        if (super.getToolTipText() == null) {
            ToolTipManager.sharedInstance().registerComponent(this);
        }
    }
    
    @Override
    void compWriteObjectNotify() {
        super.compWriteObjectNotify();
        if (super.getToolTipText() == null) {
            ToolTipManager.sharedInstance().unregisterComponent(this);
        }
    }
    
    @Override
    protected String paramString() {
        String str = (this.gridColor != null) ? this.gridColor.toString() : "";
        String str2 = this.showHorizontalLines ? "true" : "false";
        String str3 = this.showVerticalLines ? "true" : "false";
        String str4;
        if (this.autoResizeMode == 0) {
            str4 = "AUTO_RESIZE_OFF";
        }
        else if (this.autoResizeMode == 1) {
            str4 = "AUTO_RESIZE_NEXT_COLUMN";
        }
        else if (this.autoResizeMode == 2) {
            str4 = "AUTO_RESIZE_SUBSEQUENT_COLUMNS";
        }
        else if (this.autoResizeMode == 3) {
            str4 = "AUTO_RESIZE_LAST_COLUMN";
        }
        else if (this.autoResizeMode == 4) {
            str4 = "AUTO_RESIZE_ALL_COLUMNS";
        }
        else {
            str4 = "";
        }
        return super.paramString() + ",autoCreateColumnsFromModel=" + (this.autoCreateColumnsFromModel ? "true" : "false") + ",autoResizeMode=" + str4 + ",cellSelectionEnabled=" + (this.cellSelectionEnabled ? "true" : "false") + ",editingColumn=" + this.editingColumn + ",editingRow=" + this.editingRow + ",gridColor=" + str + ",preferredViewportSize=" + ((this.preferredViewportSize != null) ? this.preferredViewportSize.toString() : "") + ",rowHeight=" + this.rowHeight + ",rowMargin=" + this.rowMargin + ",rowSelectionAllowed=" + (this.rowSelectionAllowed ? "true" : "false") + ",selectionBackground=" + ((this.selectionBackground != null) ? this.selectionBackground.toString() : "") + ",selectionForeground=" + ((this.selectionForeground != null) ? this.selectionForeground.toString() : "") + ",showHorizontalLines=" + str2 + ",showVerticalLines=" + str3;
    }
    
    public boolean print() throws PrinterException {
        return this.print(JTable.PrintMode.FIT_WIDTH);
    }
    
    public boolean print(JTable.PrintMode printMode) throws PrinterException {
        return this.print(printMode, null, null);
    }
    
    public boolean print(JTable.PrintMode printMode, MessageFormat messageFormat, MessageFormat messageFormat2) throws PrinterException {
        int n = !GraphicsEnvironment.isHeadless();
        return this.print(printMode, messageFormat, messageFormat2, (boolean)n, null, (boolean)n);
    }
    
    public boolean print(JTable.PrintMode printMode, MessageFormat messageFormat, MessageFormat messageFormat2, boolean b, PrintRequestAttributeSet set, boolean b2) throws PrinterException, HeadlessException {
        return this.print(printMode, messageFormat, messageFormat2, b, set, b2, null);
    }
    
    public boolean print(JTable.PrintMode printMode, MessageFormat messageFormat, MessageFormat messageFormat2, boolean b, final PrintRequestAttributeSet set, boolean b2, PrintService printService) throws PrinterException, HeadlessException {
        if (GraphicsEnvironment.isHeadless()) {
            if (b) {
                throw new HeadlessException("Can't show print dialog.");
            }
            if (b2) {
                throw new HeadlessException("Can't run interactively.");
            }
        }
        final PrinterJob printerJob = PrinterJob.getPrinterJob();
        if (this.isEditing() && !this.getCellEditor().stopCellEditing()) {
            this.getCellEditor().cancelCellEditing();
        }
        if (set == null) {
            set = new HashPrintRequestAttributeSet();
        }
        Printable printable = this.getPrintable(printMode, messageFormat, messageFormat2);
        final PrintingStatus printingStatus;
        if (b2) {
            JTable.ThreadSafePrintable printable2 = new JTable.ThreadSafePrintable(printable);
            printingStatus = PrintingStatus.createPrintingStatus(this, printerJob);
            printable = printingStatus.createNotificationPrintable(printable2);
        }
        else {
            printingStatus = null;
        }
        printerJob.setPrintable(printable);
        if (printService != null) {
            printerJob.setPrintService(printService);
        }
        if (b && !printerJob.printDialog(set)) {
            return false;
        }
        if (!b2) {
            printerJob.print(set);
            return true;
        }
        this.printError = null;
        final Object o = new Object();
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    printerJob.print(set);
                }
                catch (Throwable t) {
                    synchronized (o) {
                        JTable.this.printError = t;
                    }
                }
                finally {
                    printingStatus.dispose();
                }
            }
        }).start();
        printingStatus.showModal(true);
        Throwable printError;
        synchronized (o) {
            printError = this.printError;
            this.printError = null;
        }
        if (printError != null) {
            if ((printError instanceof PrinterAbortException)) {
                return false;
            }
            if ((printError instanceof PrinterException)) {
                throw (PrinterException)printError;
            }
            if ((printError instanceof RuntimeException)) {
                throw (RuntimeException)printError;
            }
            if ((printError instanceof Error)) {
                throw (Error)printError;
            }
            throw new AssertionError(printError);
        }
        else {
            return true;
        }
    }
    
    public Printable getPrintable(JTable.PrintMode printMode, MessageFormat messageFormat, MessageFormat messageFormat2) {
        return new TablePrintable(this, printMode, messageFormat, messageFormat2);
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JTable.AccessibleJTable();
        }
        return this.accessibleContext;
    }
    
    static int access$200(JTable table, RowSorterEvent rowSorterEvent, int n) {
        return table.convertRowIndexToModel(rowSorterEvent, n);
    }
    
    static int[] access$300(JTable table, RowSorterEvent rowSorterEvent) {
        return table.convertSelectionToModel(rowSorterEvent);
    }
    
    static void access$500(JTable table, int[] array, int n, JTable.ModelChange modelChange) {
        table.restoreSortingSelection(array, n, modelChange);
    }
    
    public enum PrintMode
    {
        NORMAL, 
        FIT_WIDTH;
    }
    
    public static final class DropLocation extends TransferHandler.DropLocation
    {
        private final int row;
        private final int col;
        private final boolean isInsertRow;
        private final boolean isInsertCol;
        
        private DropLocation(Point point, int row, int col, boolean isInsertRow, boolean isInsertCol) {
            super(point);
            this.row = row;
            this.col = col;
            this.isInsertRow = isInsertRow;
            this.isInsertCol = isInsertCol;
        }
        
        public int getRow() {
            return this.row;
        }
        
        public int getColumn() {
            return this.col;
        }
        
        public boolean isInsertRow() {
            return this.isInsertRow;
        }
        
        public boolean isInsertColumn() {
            return this.isInsertCol;
        }
        
        @Override
        public String toString() {
            return this.getClass().getName() + "[dropPoint=" + this.getDropPoint() + ",row=" + this.row + ",column=" + this.col + ",insertRow=" + this.isInsertRow + ",insertColumn=" + this.isInsertCol + "]";
        }
    }
    
    private final class SortManager
    {
        RowSorter<? extends TableModel> sorter;
        private ListSelectionModel modelSelection;
        private int modelLeadIndex;
        private boolean syncingSelection;
        private int[] lastModelSelection;
        private SizeSequence modelRowSizes;
        
        SortManager(RowSorter<? extends TableModel> sorter) {
            (this.sorter = sorter).addRowSorterListener(JTable.this);
        }
        
        public void dispose() {
            if (this.sorter != null) {
                this.sorter.removeRowSorterListener(JTable.this);
            }
        }
        
        public void setViewRowHeight(int n, int n2) {
            if (this.modelRowSizes == null) {
                this.modelRowSizes = new SizeSequence(JTable.this.getModel().getRowCount(), JTable.this.getRowHeight());
            }
            this.modelRowSizes.setSize(JTable.this.convertRowIndexToModel(n), n2);
        }
        
        public void allChanged() {
            this.modelLeadIndex = -1;
            this.modelSelection = null;
            this.modelRowSizes = null;
        }
        
        public void viewSelectionChanged(ListSelectionEvent listSelectionEvent) {
            if (!this.syncingSelection && this.modelSelection != null) {
                this.modelSelection = null;
            }
        }
        
        public void prepareForChange(RowSorterEvent rowSorterEvent, JTable.ModelChange modelChange) {
            if (JTable.this.getUpdateSelectionOnSort()) {
                this.cacheSelection(rowSorterEvent, modelChange);
            }
        }
        
        private void cacheSelection(RowSorterEvent rowSorterEvent, JTable.ModelChange modelChange) {
            if (rowSorterEvent != null) {
                if (this.modelSelection == null && this.sorter.getViewRowCount() != JTable.this.getModel().getRowCount()) {
                    this.modelSelection = new DefaultListSelectionModel();
                    ListSelectionModel selectionModel = JTable.this.getSelectionModel();
                    int i;
                    int maxSelectionIndex;
                    int access$200;
                    for (i = selectionModel.getMinSelectionIndex(), maxSelectionIndex = selectionModel.getMaxSelectionIndex(), ; i <= maxSelectionIndex; ++i) {
                        if (selectionModel.isSelectedIndex(i)) {
                            access$200 = JTable.access$200(JTable.this, rowSorterEvent, i);
                            if (access$200 != -1) {
                                this.modelSelection.addSelectionInterval(access$200, access$200);
                            }
                        }
                    }
                    int access$201 = JTable.access$200(JTable.this, rowSorterEvent, selectionModel.getLeadSelectionIndex());
                    SwingUtilities2.setLeadAnchorWithoutSelection(this.modelSelection, access$201, access$201);
                }
                else if (this.modelSelection == null) {
                    this.cacheModelSelection(rowSorterEvent);
                }
            }
            else if (modelChange.allRowsChanged) {
                this.modelSelection = null;
            }
            else if (this.modelSelection != null) {
                switch (modelChange.type) {
                    case -1: {
                        this.modelSelection.removeIndexInterval(modelChange.startModelIndex, modelChange.endModelIndex);
                        break;
                    }
                    case 1: {
                        this.modelSelection.insertIndexInterval(modelChange.startModelIndex, modelChange.length, true);
                        break;
                    }
                    default: {
                        break;
                    }
                }
            }
            else {
                this.cacheModelSelection(null);
            }
        }
        
        private void cacheModelSelection(RowSorterEvent rowSorterEvent) {
            this.lastModelSelection = JTable.access$300(JTable.this, rowSorterEvent);
            this.modelLeadIndex = JTable.access$200(JTable.this, rowSorterEvent, JTable.this.selectionModel.getLeadSelectionIndex());
        }
        
        public void processChange(RowSorterEvent rowSorterEvent, JTable.ModelChange modelChange, boolean b) {
            if (modelChange != null) {
                if (modelChange.allRowsChanged) {
                    this.modelRowSizes = null;
                    JTable.this.rowModel = null;
                }
                else if (this.modelRowSizes != null) {
                    if (modelChange.type == 1) {
                        this.modelRowSizes.insertEntries(modelChange.startModelIndex, modelChange.endModelIndex - modelChange.startModelIndex + 1, JTable.this.getRowHeight());
                    }
                    else if (modelChange.type == -1) {
                        this.modelRowSizes.removeEntries(modelChange.startModelIndex, modelChange.endModelIndex - modelChange.startModelIndex + 1);
                    }
                }
            }
            if (b) {
                this.setViewRowHeightsFromModel();
                this.restoreSelection(modelChange);
            }
        }
        
        private void setViewRowHeightsFromModel() {
            if (this.modelRowSizes != null) {
                JTable.this.rowModel.setSizes(JTable.this.getRowCount(), JTable.this.getRowHeight());
                for (int i = JTable.this.getRowCount() - 1; i >= 0; --i) {
                    JTable.this.rowModel.setSize(i, this.modelRowSizes.getSize(JTable.this.convertRowIndexToModel(i)));
                }
            }
        }
        
        private void restoreSelection(JTable.ModelChange modelChange) {
            this.syncingSelection = true;
            if (this.lastModelSelection != null) {
                JTable.access$500(JTable.this, this.lastModelSelection, this.modelLeadIndex, modelChange);
                this.lastModelSelection = null;
            }
            else if (this.modelSelection != null) {
                ListSelectionModel selectionModel = JTable.this.getSelectionModel();
                selectionModel.setValueIsAdjusting(true);
                selectionModel.clearSelection();
                int i;
                int maxSelectionIndex;
                int convertRowIndexToView;
                for (i = this.modelSelection.getMinSelectionIndex(), maxSelectionIndex = this.modelSelection.getMaxSelectionIndex(), ; i <= maxSelectionIndex; ++i) {
                    if (this.modelSelection.isSelectedIndex(i)) {
                        convertRowIndexToView = JTable.this.convertRowIndexToView(i);
                        if (convertRowIndexToView != -1) {
                            selectionModel.addSelectionInterval(convertRowIndexToView, convertRowIndexToView);
                        }
                    }
                }
                int n = this.modelSelection.getLeadSelectionIndex();
                if (n != -1 && !this.modelSelection.isSelectionEmpty()) {
                    n = JTable.this.convertRowIndexToView(n);
                }
                SwingUtilities2.setLeadAnchorWithoutSelection(selectionModel, n, n);
                selectionModel.setValueIsAdjusting(false);
            }
            this.syncingSelection = false;
        }
    }
    
    private final class ModelChange
    {
        int startModelIndex;
        int endModelIndex;
        int type;
        int modelRowCount;
        TableModelEvent event;
        int length;
        boolean allRowsChanged;
        
        ModelChange(TableModelEvent event) {
            this.startModelIndex = Math.max(0, event.getFirstRow());
            this.endModelIndex = event.getLastRow();
            this.modelRowCount = JTable.this.getModel().getRowCount();
            if (this.endModelIndex < 0) {
                this.endModelIndex = Math.max(0, this.modelRowCount - 1);
            }
            this.length = this.endModelIndex - this.startModelIndex + 1;
            this.type = event.getType();
            this.event = event;
            this.allRowsChanged = (event.getLastRow() == Integer.MAX_VALUE);
        }
    }
    
    static class NumberRenderer extends DefaultTableCellRenderer.UIResource
    {
        public NumberRenderer() {
            this.setHorizontalAlignment(4);
        }
    }
    
    static class DoubleRenderer extends JTable.NumberRenderer
    {
        NumberFormat formatter;
        
        public DoubleRenderer() {
        }
        
        public void setValue(Object obj) {
            if (this.formatter == null) {
                this.formatter = NumberFormat.getInstance();
            }
            this.setText((obj == null) ? "" : this.formatter.format(obj));
        }
    }
    
    static class DateRenderer extends DefaultTableCellRenderer.UIResource
    {
        DateFormat formatter;
        
        public DateRenderer() {
        }
        
        public void setValue(Object obj) {
            if (this.formatter == null) {
                this.formatter = DateFormat.getDateInstance();
            }
            this.setText((obj == null) ? "" : this.formatter.format(obj));
        }
    }
    
    static class IconRenderer extends DefaultTableCellRenderer.UIResource
    {
        public IconRenderer() {
            this.setHorizontalAlignment(0);
        }
        
        public void setValue(Object o) {
            this.setIcon((o instanceof Icon) ? ((Icon)o) : null);
        }
    }
    
    static class BooleanRenderer extends JCheckBox implements TableCellRenderer, UIResource
    {
        private static final Border noFocusBorder;
        
        public BooleanRenderer() {
            this.setHorizontalAlignment(0);
            this.setBorderPainted(true);
        }
        
        @Override
        public Component getTableCellRendererComponent(JTable table, Object o, boolean b, boolean b2, int n, int n2) {
            if (b) {
                this.setForeground(table.getSelectionForeground());
                super.setBackground(table.getSelectionBackground());
            }
            else {
                this.setForeground(table.getForeground());
                this.setBackground(table.getBackground());
            }
            this.setSelected(o != null && (boolean)o);
            if (b2) {
                this.setBorder(UIManager.getBorder("Table.focusCellHighlightBorder"));
                return this;
            }
            this.setBorder(JTable.BooleanRenderer.noFocusBorder);
            return this;
        }
        
        static {
            noFocusBorder = new EmptyBorder(1, 1, 1, 1);
        }
    }
    
    static class GenericEditor extends DefaultCellEditor
    {
        Class[] argTypes;
        Constructor constructor;
        Object value;
        
        public GenericEditor() {
            super(new JTextField());
            this.argTypes = new Class[] { String.class };
            this.getComponent().setName("Table.editor");
        }
        
        @Override
        public boolean stopCellEditing() {
            String s = (String)super.getCellEditorValue();
            try {
                if ("".equals(s)) {
                    if (this.constructor.getDeclaringClass() == String.class) {
                        this.value = s;
                    }
                    return super.stopCellEditing();
                }
                else {
                    SwingUtilities2.checkAccess(this.constructor.getModifiers());
                    this.value = this.constructor.newInstance(new Object[] { s });
                }
            }
            catch (Exception ex) {
                ((JComponent)this.getComponent()).setBorder(new LineBorder(Color.red));
                return false;
            }
            return super.stopCellEditing();
        }
        
        @Override
        public Component getTableCellEditorComponent(JTable table, Object o, boolean b, int n, int n2) {
            this.value = null;
            ((JComponent)this.getComponent()).setBorder(new LineBorder(Color.black));
            try {
                Class<Object> columnClass = table.getColumnClass(n2);
                if (columnClass == Object.class) {
                    columnClass = String.class;
                }
                ReflectUtil.checkPackageAccess(columnClass);
                SwingUtilities2.checkAccess(columnClass.getModifiers());
                this.constructor = columnClass.getConstructor(this.argTypes);
            }
            catch (Exception ex) {
                return null;
            }
            return super.getTableCellEditorComponent(table, o, b, n, n2);
        }
        
        @Override
        public Object getCellEditorValue() {
            return this.value;
        }
    }
    
    static class NumberEditor extends JTable.GenericEditor
    {
        public NumberEditor() {
            ((JTextField)this.getComponent()).setHorizontalAlignment(4);
        }
    }
    
    static class BooleanEditor extends DefaultCellEditor
    {
        public BooleanEditor() {
            super(new JCheckBox());
            ((JCheckBox)this.getComponent()).setHorizontalAlignment(0);
        }
    }
    
    class CellEditorRemover implements PropertyChangeListener
    {
        KeyboardFocusManager focusManager;
        
        public CellEditorRemover(KeyboardFocusManager focusManager) {
            this.focusManager = focusManager;
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            if (!JTable.this.isEditing() || JTable.this.getClientProperty("terminateEditOnFocusLost") != Boolean.TRUE) {
                return;
            }
            Component component = this.focusManager.getPermanentFocusOwner();
            while (component != null) {
                if (component == JTable.this) {
                    return;
                }
                if (component instanceof Window || (component instanceof Applet && component.getParent() == null)) {
                    if (component == SwingUtilities.getRoot(JTable.this) && !JTable.this.getCellEditor().stopCellEditing()) {
                        JTable.this.getCellEditor().cancelCellEditing();
                        break;
                    }
                    break;
                }
                else {
                    component = component.getParent();
                }
            }
            return;
        }
    }
    
    private class ThreadSafePrintable implements Printable
    {
        private Printable printDelegate;
        private int retVal;
        private Throwable retThrowable;
        
        public ThreadSafePrintable(Printable printDelegate) {
            this.printDelegate = printDelegate;
        }
        
        @Override
        public int print(Graphics graphics, PageFormat pageFormat, int n) throws PrinterException {
            Runnable runnable = new Runnable() {
                @Override
                public synchronized void run() {
                    try {
                        JTable.ThreadSafePrintable.this.retVal = JTable.ThreadSafePrintable.this.printDelegate.print(graphics, pageFormat, n);
                    }
                    catch (Throwable t) {
                        JTable.ThreadSafePrintable.this.retThrowable = t;
                    }
                    finally {
                        this.notifyAll();
                    }
                }
            };
            synchronized (runnable) {
                this.retVal = -1;
                this.retThrowable = null;
                SwingUtilities.invokeLater(runnable);
                while (this.retVal == -1 && this.retThrowable == null) {
                    try {
                        runnable.wait();
                    }
                    catch (InterruptedException ex) {}
                }
                if (this.retThrowable != null) {
                    if ((this.retThrowable instanceof PrinterException)) {
                        throw (PrinterException)this.retThrowable;
                    }
                    if ((this.retThrowable instanceof RuntimeException)) {
                        throw (RuntimeException)this.retThrowable;
                    }
                    if ((this.retThrowable instanceof Error)) {
                        throw (Error)this.retThrowable;
                    }
                    throw new AssertionError(this.retThrowable);
                }
                else {
                    return this.retVal;
                }
            }
        }
    }
    
    protected class AccessibleJTable extends JComponent.AccessibleJComponent implements AccessibleSelection, ListSelectionListener, TableModelListener, TableColumnModelListener, CellEditorListener, PropertyChangeListener, AccessibleExtendedTable
    {
        int previousFocusedRow;
        int previousFocusedCol;
        private Accessible caption;
        private Accessible summary;
        private Accessible[] rowDescription;
        private Accessible[] columnDescription;
        
        protected AccessibleJTable() {
            JTable.this.addPropertyChangeListener(this);
            JTable.this.getSelectionModel().addListSelectionListener(this);
            TableColumnModel columnModel = JTable.this.getColumnModel();
            columnModel.addColumnModelListener(this);
            columnModel.getSelectionModel().addListSelectionListener(this);
            JTable.this.getModel().addTableModelListener(this);
            this.previousFocusedRow = JTable.this.getSelectionModel().getLeadSelectionIndex();
            this.previousFocusedCol = JTable.this.getColumnModel().getSelectionModel().getLeadSelectionIndex();
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            String propertyName = propertyChangeEvent.getPropertyName();
            Object oldValue = propertyChangeEvent.getOldValue();
            Object newValue = propertyChangeEvent.getNewValue();
            if (propertyName.compareTo("model") == 0) {
                if (oldValue != null && oldValue instanceof TableModel) {
                    ((TableModel)oldValue).removeTableModelListener(this);
                }
                if (newValue != null && newValue instanceof TableModel) {
                    ((TableModel)newValue).addTableModelListener(this);
                }
            }
            else if (propertyName.compareTo("selectionModel") == 0) {
                Object source = propertyChangeEvent.getSource();
                if (source == JTable.this) {
                    if (oldValue != null && oldValue instanceof ListSelectionModel) {
                        ((ListSelectionModel)oldValue).removeListSelectionListener(this);
                    }
                    if (newValue != null && newValue instanceof ListSelectionModel) {
                        ((ListSelectionModel)newValue).addListSelectionListener(this);
                    }
                }
                else if (source == JTable.this.getColumnModel()) {
                    if (oldValue != null && oldValue instanceof ListSelectionModel) {
                        ((ListSelectionModel)oldValue).removeListSelectionListener(this);
                    }
                    if (newValue != null && newValue instanceof ListSelectionModel) {
                        ((ListSelectionModel)newValue).addListSelectionListener(this);
                    }
                }
            }
            else if (propertyName.compareTo("columnModel") == 0) {
                if (oldValue != null && oldValue instanceof TableColumnModel) {
                    TableColumnModel tableColumnModel = (TableColumnModel)oldValue;
                    tableColumnModel.removeColumnModelListener(this);
                    tableColumnModel.getSelectionModel().removeListSelectionListener(this);
                }
                if (newValue != null && newValue instanceof TableColumnModel) {
                    TableColumnModel tableColumnModel2 = (TableColumnModel)newValue;
                    tableColumnModel2.addColumnModelListener(this);
                    tableColumnModel2.getSelectionModel().addListSelectionListener(this);
                }
            }
            else if (propertyName.compareTo("tableCellEditor") == 0) {
                if (oldValue != null && oldValue instanceof TableCellEditor) {
                    ((TableCellEditor)oldValue).removeCellEditorListener(this);
                }
                if (newValue != null && newValue instanceof TableCellEditor) {
                    ((TableCellEditor)newValue).addCellEditorListener(this);
                }
            }
        }
        
        @Override
        public void tableChanged(TableModelEvent tableModelEvent) {
            this.firePropertyChange("AccessibleVisibleData", null, null);
            if (tableModelEvent != null) {
                int column = tableModelEvent.getColumn();
                int column2 = tableModelEvent.getColumn();
                if (column == -1) {
                    column = 0;
                    column2 = JTable.this.getColumnCount() - 1;
                }
                this.firePropertyChange("accessibleTableModelChanged", null, new JTable.AccessibleJTable.AccessibleJTableModelChange(tableModelEvent.getType(), tableModelEvent.getFirstRow(), tableModelEvent.getLastRow(), column, column2));
            }
        }
        
        public void tableRowsInserted(TableModelEvent tableModelEvent) {
            this.firePropertyChange("AccessibleVisibleData", null, null);
            int column = tableModelEvent.getColumn();
            int column2 = tableModelEvent.getColumn();
            if (column == -1) {
                column = 0;
                column2 = JTable.this.getColumnCount() - 1;
            }
            this.firePropertyChange("accessibleTableModelChanged", null, new JTable.AccessibleJTable.AccessibleJTableModelChange(tableModelEvent.getType(), tableModelEvent.getFirstRow(), tableModelEvent.getLastRow(), column, column2));
        }
        
        public void tableRowsDeleted(TableModelEvent tableModelEvent) {
            this.firePropertyChange("AccessibleVisibleData", null, null);
            int column = tableModelEvent.getColumn();
            int column2 = tableModelEvent.getColumn();
            if (column == -1) {
                column = 0;
                column2 = JTable.this.getColumnCount() - 1;
            }
            this.firePropertyChange("accessibleTableModelChanged", null, new JTable.AccessibleJTable.AccessibleJTableModelChange(tableModelEvent.getType(), tableModelEvent.getFirstRow(), tableModelEvent.getLastRow(), column, column2));
        }
        
        @Override
        public void columnAdded(TableColumnModelEvent tableColumnModelEvent) {
            this.firePropertyChange("AccessibleVisibleData", null, null);
            this.firePropertyChange("accessibleTableModelChanged", null, new JTable.AccessibleJTable.AccessibleJTableModelChange(1, 0, 0, tableColumnModelEvent.getFromIndex(), tableColumnModelEvent.getToIndex()));
        }
        
        @Override
        public void columnRemoved(TableColumnModelEvent tableColumnModelEvent) {
            this.firePropertyChange("AccessibleVisibleData", null, null);
            this.firePropertyChange("accessibleTableModelChanged", null, new JTable.AccessibleJTable.AccessibleJTableModelChange(-1, 0, 0, tableColumnModelEvent.getFromIndex(), tableColumnModelEvent.getToIndex()));
        }
        
        @Override
        public void columnMoved(TableColumnModelEvent tableColumnModelEvent) {
            this.firePropertyChange("AccessibleVisibleData", null, null);
            this.firePropertyChange("accessibleTableModelChanged", null, new JTable.AccessibleJTable.AccessibleJTableModelChange(-1, 0, 0, tableColumnModelEvent.getFromIndex(), tableColumnModelEvent.getFromIndex()));
            this.firePropertyChange("accessibleTableModelChanged", null, new JTable.AccessibleJTable.AccessibleJTableModelChange(1, 0, 0, tableColumnModelEvent.getToIndex(), tableColumnModelEvent.getToIndex()));
        }
        
        @Override
        public void columnMarginChanged(ChangeEvent changeEvent) {
            this.firePropertyChange("AccessibleVisibleData", null, null);
        }
        
        @Override
        public void columnSelectionChanged(ListSelectionEvent listSelectionEvent) {
        }
        
        @Override
        public void editingStopped(ChangeEvent changeEvent) {
            this.firePropertyChange("AccessibleVisibleData", null, null);
        }
        
        @Override
        public void editingCanceled(ChangeEvent changeEvent) {
        }
        
        @Override
        public void valueChanged(ListSelectionEvent listSelectionEvent) {
            this.firePropertyChange("AccessibleSelection", Boolean.valueOf(false), Boolean.valueOf(true));
            int leadSelectionIndex = JTable.this.getSelectionModel().getLeadSelectionIndex();
            int leadSelectionIndex2 = JTable.this.getColumnModel().getSelectionModel().getLeadSelectionIndex();
            if (leadSelectionIndex != this.previousFocusedRow || leadSelectionIndex2 != this.previousFocusedCol) {
                this.firePropertyChange("AccessibleActiveDescendant", this.getAccessibleAt(this.previousFocusedRow, this.previousFocusedCol), this.getAccessibleAt(leadSelectionIndex, leadSelectionIndex2));
                this.previousFocusedRow = leadSelectionIndex;
                this.previousFocusedCol = leadSelectionIndex2;
            }
        }
        
        @Override
        public AccessibleSelection getAccessibleSelection() {
            return this;
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.TABLE;
        }
        
        @Override
        public Accessible getAccessibleAt(Point point) {
            int columnAtPoint = JTable.this.columnAtPoint(point);
            int rowAtPoint = JTable.this.rowAtPoint(point);
            if (columnAtPoint != -1 && rowAtPoint != -1) {
                TableCellRenderer tableCellRenderer = JTable.this.getColumnModel().getColumn(columnAtPoint).getCellRenderer();
                if (tableCellRenderer == null) {
                    tableCellRenderer = JTable.this.getDefaultRenderer(JTable.this.getColumnClass(columnAtPoint));
                }
                tableCellRenderer.getTableCellRendererComponent(JTable.this, null, false, false, rowAtPoint, columnAtPoint);
                return new JTable.AccessibleJTable.AccessibleJTableCell(JTable.this, rowAtPoint, columnAtPoint, this.getAccessibleIndexAt(rowAtPoint, columnAtPoint));
            }
            else {
                return null;
            }
        }
        
        @Override
        public int getAccessibleChildrenCount() {
            return JTable.this.getColumnCount() * JTable.this.getRowCount();
        }
        
        @Override
        public Accessible getAccessibleChild(int n) {
            if (n < 0 || n >= this.getAccessibleChildrenCount()) {
                return null;
            }
            int accessibleColumnAtIndex = this.getAccessibleColumnAtIndex(n);
            int accessibleRowAtIndex = this.getAccessibleRowAtIndex(n);
            TableCellRenderer tableCellRenderer = JTable.this.getColumnModel().getColumn(accessibleColumnAtIndex).getCellRenderer();
            if (tableCellRenderer == null) {
                tableCellRenderer = JTable.this.getDefaultRenderer(JTable.this.getColumnClass(accessibleColumnAtIndex));
            }
            tableCellRenderer.getTableCellRendererComponent(JTable.this, null, false, false, accessibleRowAtIndex, accessibleColumnAtIndex);
            return new JTable.AccessibleJTable.AccessibleJTableCell(JTable.this, accessibleRowAtIndex, accessibleColumnAtIndex, this.getAccessibleIndexAt(accessibleRowAtIndex, accessibleColumnAtIndex));
        }
        
        @Override
        public int getAccessibleSelectionCount() {
            int selectedRowCount = JTable.this.getSelectedRowCount();
            int selectedColumnCount = JTable.this.getSelectedColumnCount();
            if (JTable.this.cellSelectionEnabled) {
                return selectedRowCount * selectedColumnCount;
            }
            if (JTable.this.getRowSelectionAllowed() && JTable.this.getColumnSelectionAllowed()) {
                return selectedRowCount * JTable.this.getColumnCount() + selectedColumnCount * JTable.this.getRowCount() - selectedRowCount * selectedColumnCount;
            }
            if (JTable.this.getRowSelectionAllowed()) {
                return selectedRowCount * JTable.this.getColumnCount();
            }
            if (JTable.this.getColumnSelectionAllowed()) {
                return selectedColumnCount * JTable.this.getRowCount();
            }
            return 0;
        }
        
        @Override
        public Accessible getAccessibleSelection(int n) {
            if (n < 0 || n > this.getAccessibleSelectionCount()) {
                return null;
            }
            JTable.this.getSelectedRowCount();
            int selectedColumnCount = JTable.this.getSelectedColumnCount();
            int[] selectedRows = JTable.this.getSelectedRows();
            int[] selectedColumns = JTable.this.getSelectedColumns();
            int columnCount = JTable.this.getColumnCount();
            int rowCount = JTable.this.getRowCount();
            if (JTable.this.cellSelectionEnabled) {
                return this.getAccessibleChild(selectedRows[n / selectedColumnCount] * columnCount + selectedColumns[n % selectedColumnCount]);
            }
            if (JTable.this.getRowSelectionAllowed() && JTable.this.getColumnSelectionAllowed()) {
                int n2 = selectedRows[0] != 0;
                int i = 0;
                int n3 = -1;
                while (i < selectedRows.length) {
                    switch (n2) {
                        case 0: {
                            if (n < columnCount) {
                                return this.getAccessibleChild(selectedRows[i] * columnCount + n % columnCount);
                            }
                            n -= columnCount;
                            if (i + 1 == selectedRows.length || selectedRows[i] != selectedRows[i + 1] - 1) {
                                n2 = 1;
                                n3 = selectedRows[i];
                            }
                            ++i;
                            continue;
                        }
                        case 1: {
                            if (n < selectedColumnCount * (selectedRows[i] - ((n3 == -1) ? 0 : (n3 + 1)))) {
                                return this.getAccessibleChild((((i > 0) ? (selectedRows[i - 1] + 1) : 0) + n / selectedColumnCount) * columnCount + selectedColumns[n % selectedColumnCount]);
                            }
                            n -= selectedColumnCount * (selectedRows[i] - ((n3 == -1) ? 0 : (n3 + 1)));
                            n2 = 0;
                            continue;
                        }
                    }
                }
                if (n < selectedColumnCount * (rowCount - ((n3 == -1) ? 0 : (n3 + 1)))) {
                    return this.getAccessibleChild((selectedRows[i - 1] + n / selectedColumnCount + 1) * columnCount + selectedColumns[n % selectedColumnCount]);
                }
            }
            else {
                if (JTable.this.getRowSelectionAllowed()) {
                    return this.getAccessibleChild(selectedRows[n / columnCount] * columnCount + n % columnCount);
                }
                if (JTable.this.getColumnSelectionAllowed()) {
                    return this.getAccessibleChild(n / selectedColumnCount * columnCount + selectedColumns[n % selectedColumnCount]);
                }
            }
            return null;
        }
        
        @Override
        public boolean isAccessibleChildSelected(int n) {
            return JTable.this.isCellSelected(this.getAccessibleRowAtIndex(n), this.getAccessibleColumnAtIndex(n));
        }
        
        @Override
        public void addAccessibleSelection(int n) {
            JTable.this.changeSelection(this.getAccessibleRowAtIndex(n), this.getAccessibleColumnAtIndex(n), true, false);
        }
        
        @Override
        public void removeAccessibleSelection(int n) {
            if (JTable.this.cellSelectionEnabled) {
                int accessibleColumnAtIndex = this.getAccessibleColumnAtIndex(n);
                int accessibleRowAtIndex = this.getAccessibleRowAtIndex(n);
                JTable.this.removeRowSelectionInterval(accessibleRowAtIndex, accessibleRowAtIndex);
                JTable.this.removeColumnSelectionInterval(accessibleColumnAtIndex, accessibleColumnAtIndex);
            }
        }
        
        @Override
        public void clearAccessibleSelection() {
            JTable.this.clearSelection();
        }
        
        @Override
        public void selectAllAccessibleSelection() {
            if (JTable.this.cellSelectionEnabled) {
                JTable.this.selectAll();
            }
        }
        
        @Override
        public int getAccessibleRow(int n) {
            return this.getAccessibleRowAtIndex(n);
        }
        
        @Override
        public int getAccessibleColumn(int n) {
            return this.getAccessibleColumnAtIndex(n);
        }
        
        @Override
        public int getAccessibleIndex(int n, int n2) {
            return this.getAccessibleIndexAt(n, n2);
        }
        
        @Override
        public AccessibleTable getAccessibleTable() {
            return this;
        }
        
        @Override
        public Accessible getAccessibleCaption() {
            return this.caption;
        }
        
        @Override
        public void setAccessibleCaption(Accessible caption) {
            this.firePropertyChange("accessibleTableCaptionChanged", this.caption, this.caption = caption);
        }
        
        @Override
        public Accessible getAccessibleSummary() {
            return this.summary;
        }
        
        @Override
        public void setAccessibleSummary(Accessible summary) {
            this.firePropertyChange("accessibleTableSummaryChanged", this.summary, this.summary = summary);
        }
        
        @Override
        public int getAccessibleRowCount() {
            return JTable.this.getRowCount();
        }
        
        @Override
        public int getAccessibleColumnCount() {
            return JTable.this.getColumnCount();
        }
        
        @Override
        public Accessible getAccessibleAt(int n, int n2) {
            return this.getAccessibleChild(n * this.getAccessibleColumnCount() + n2);
        }
        
        @Override
        public int getAccessibleRowExtentAt(int n, int n2) {
            return 1;
        }
        
        @Override
        public int getAccessibleColumnExtentAt(int n, int n2) {
            return 1;
        }
        
        @Override
        public AccessibleTable getAccessibleRowHeader() {
            return null;
        }
        
        @Override
        public void setAccessibleRowHeader(AccessibleTable accessibleTable) {
        }
        
        @Override
        public AccessibleTable getAccessibleColumnHeader() {
            JTableHeader tableHeader = JTable.this.getTableHeader();
            return (tableHeader == null) ? null : new JTable.AccessibleJTable.AccessibleTableHeader(tableHeader);
        }
        
        @Override
        public void setAccessibleColumnHeader(AccessibleTable accessibleTable) {
        }
        
        @Override
        public Accessible getAccessibleRowDescription(int i) {
            if (i < 0 || i >= this.getAccessibleRowCount()) {
                throw new IllegalArgumentException(Integer.toString(i));
            }
            if (this.rowDescription == null) {
                return null;
            }
            return this.rowDescription[i];
        }
        
        @Override
        public void setAccessibleRowDescription(int i, Accessible accessible) {
            if (i < 0 || i >= this.getAccessibleRowCount()) {
                throw new IllegalArgumentException(Integer.toString(i));
            }
            if (this.rowDescription == null) {
                this.rowDescription = new Accessible[this.getAccessibleRowCount()];
            }
            this.rowDescription[i] = accessible;
            return;
        }
        
        @Override
        public Accessible getAccessibleColumnDescription(int i) {
            if (i < 0 || i >= this.getAccessibleColumnCount()) {
                throw new IllegalArgumentException(Integer.toString(i));
            }
            if (this.columnDescription == null) {
                return null;
            }
            return this.columnDescription[i];
        }
        
        @Override
        public void setAccessibleColumnDescription(int i, Accessible accessible) {
            if (i < 0 || i >= this.getAccessibleColumnCount()) {
                throw new IllegalArgumentException(Integer.toString(i));
            }
            if (this.columnDescription == null) {
                this.columnDescription = new Accessible[this.getAccessibleColumnCount()];
            }
            this.columnDescription[i] = accessible;
            return;
        }
        
        @Override
        public boolean isAccessibleSelected(int n, int n2) {
            return JTable.this.isCellSelected(n, n2);
        }
        
        @Override
        public boolean isAccessibleRowSelected(int n) {
            return JTable.this.isRowSelected(n);
        }
        
        @Override
        public boolean isAccessibleColumnSelected(int n) {
            return JTable.this.isColumnSelected(n);
        }
        
        @Override
        public int[] getSelectedAccessibleRows() {
            return JTable.this.getSelectedRows();
        }
        
        @Override
        public int[] getSelectedAccessibleColumns() {
            return JTable.this.getSelectedColumns();
        }
        
        public int getAccessibleRowAtIndex(int n) {
            int accessibleColumnCount = this.getAccessibleColumnCount();
            if (accessibleColumnCount == 0) {
                return -1;
            }
            return n / accessibleColumnCount;
        }
        
        public int getAccessibleColumnAtIndex(int n) {
            int accessibleColumnCount = this.getAccessibleColumnCount();
            if (accessibleColumnCount == 0) {
                return -1;
            }
            return n % accessibleColumnCount;
        }
        
        public int getAccessibleIndexAt(int n, int n2) {
            return n * this.getAccessibleColumnCount() + n2;
        }
        
        protected class AccessibleJTableModelChange implements AccessibleTableModelChange
        {
            protected int type;
            protected int firstRow;
            protected int lastRow;
            protected int firstColumn;
            protected int lastColumn;
            
            protected AccessibleJTableModelChange(int type, int firstRow, int lastRow, int firstColumn, int lastColumn) {
                this.type = type;
                this.firstRow = firstRow;
                this.lastRow = lastRow;
                this.firstColumn = firstColumn;
                this.lastColumn = lastColumn;
            }
            
            @Override
            public int getType() {
                return this.type;
            }
            
            @Override
            public int getFirstRow() {
                return this.firstRow;
            }
            
            @Override
            public int getLastRow() {
                return this.lastRow;
            }
            
            @Override
            public int getFirstColumn() {
                return this.firstColumn;
            }
            
            @Override
            public int getLastColumn() {
                return this.lastColumn;
            }
        }
        
        private class AccessibleTableHeader implements AccessibleTable
        {
            private JTableHeader header;
            private TableColumnModel headerModel;
            
            AccessibleTableHeader(JTableHeader header) {
                this.header = header;
                this.headerModel = header.getColumnModel();
            }
            
            @Override
            public Accessible getAccessibleCaption() {
                return null;
            }
            
            @Override
            public void setAccessibleCaption(Accessible accessible) {
            }
            
            @Override
            public Accessible getAccessibleSummary() {
                return null;
            }
            
            @Override
            public void setAccessibleSummary(Accessible accessible) {
            }
            
            @Override
            public int getAccessibleRowCount() {
                return 1;
            }
            
            @Override
            public int getAccessibleColumnCount() {
                return this.headerModel.getColumnCount();
            }
            
            @Override
            public Accessible getAccessibleAt(int n, int n2) {
                TableColumn column = this.headerModel.getColumn(n2);
                TableCellRenderer tableCellRenderer = column.getHeaderRenderer();
                if (tableCellRenderer == null) {
                    tableCellRenderer = this.header.getDefaultRenderer();
                }
                return new JTable.AccessibleJTable.AccessibleJTableHeaderCell(n, n2, JTable.this.getTableHeader(), tableCellRenderer.getTableCellRendererComponent(this.header.getTable(), column.getHeaderValue(), false, false, -1, n2));
            }
            
            @Override
            public int getAccessibleRowExtentAt(int n, int n2) {
                return 1;
            }
            
            @Override
            public int getAccessibleColumnExtentAt(int n, int n2) {
                return 1;
            }
            
            @Override
            public AccessibleTable getAccessibleRowHeader() {
                return null;
            }
            
            @Override
            public void setAccessibleRowHeader(AccessibleTable accessibleTable) {
            }
            
            @Override
            public AccessibleTable getAccessibleColumnHeader() {
                return null;
            }
            
            @Override
            public void setAccessibleColumnHeader(AccessibleTable accessibleTable) {
            }
            
            @Override
            public Accessible getAccessibleRowDescription(int n) {
                return null;
            }
            
            @Override
            public void setAccessibleRowDescription(int n, Accessible accessible) {
            }
            
            @Override
            public Accessible getAccessibleColumnDescription(int n) {
                return null;
            }
            
            @Override
            public void setAccessibleColumnDescription(int n, Accessible accessible) {
            }
            
            @Override
            public boolean isAccessibleSelected(int n, int n2) {
                return false;
            }
            
            @Override
            public boolean isAccessibleRowSelected(int n) {
                return false;
            }
            
            @Override
            public boolean isAccessibleColumnSelected(int n) {
                return false;
            }
            
            @Override
            public int[] getSelectedAccessibleRows() {
                return new int[0];
            }
            
            @Override
            public int[] getSelectedAccessibleColumns() {
                return new int[0];
            }
        }
        
        protected class AccessibleJTableCell extends AccessibleContext implements Accessible, AccessibleComponent
        {
            private JTable parent;
            private int row;
            private int column;
            private int index;
            
            public AccessibleJTableCell(JTable parent, int row, int column, int index) {
                this.parent = parent;
                this.row = row;
                this.column = column;
                this.index = index;
                this.setAccessibleParent(this.parent);
            }
            
            @Override
            public AccessibleContext getAccessibleContext() {
                return this;
            }
            
            protected AccessibleContext getCurrentAccessibleContext() {
                TableCellRenderer tableCellRenderer = JTable.this.getColumnModel().getColumn(this.column).getCellRenderer();
                if (tableCellRenderer == null) {
                    tableCellRenderer = JTable.this.getDefaultRenderer(JTable.this.getColumnClass(this.column));
                }
                Component tableCellRendererComponent = tableCellRenderer.getTableCellRendererComponent(JTable.this, JTable.this.getValueAt(this.row, this.column), false, false, this.row, this.column);
                if ((tableCellRendererComponent instanceof Accessible)) {
                    return tableCellRendererComponent.getAccessibleContext();
                }
                return null;
            }
            
            protected Component getCurrentComponent() {
                TableCellRenderer tableCellRenderer = JTable.this.getColumnModel().getColumn(this.column).getCellRenderer();
                if (tableCellRenderer == null) {
                    tableCellRenderer = JTable.this.getDefaultRenderer(JTable.this.getColumnClass(this.column));
                }
                return tableCellRenderer.getTableCellRendererComponent(JTable.this, null, false, false, this.row, this.column);
            }
            
            @Override
            public String getAccessibleName() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    String accessibleName = currentAccessibleContext.getAccessibleName();
                    if (accessibleName != null && accessibleName != "") {
                        return accessibleName;
                    }
                }
                if (this.accessibleName != null && this.accessibleName != "") {
                    return this.accessibleName;
                }
                return (String)JTable.this.getClientProperty("AccessibleName");
            }
            
            @Override
            public void setAccessibleName(String s) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.setAccessibleName(s);
                }
                else {
                    super.setAccessibleName(s);
                }
            }
            
            @Override
            public String getAccessibleDescription() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleDescription();
                }
                return super.getAccessibleDescription();
            }
            
            @Override
            public void setAccessibleDescription(String s) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.setAccessibleDescription(s);
                }
                else {
                    super.setAccessibleDescription(s);
                }
            }
            
            @Override
            public AccessibleRole getAccessibleRole() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleRole();
                }
                return AccessibleRole.UNKNOWN;
            }
            
            @Override
            public AccessibleStateSet getAccessibleStateSet() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                Object accessibleStateSet = null;
                if (currentAccessibleContext != null) {
                    accessibleStateSet = currentAccessibleContext.getAccessibleStateSet();
                }
                if (accessibleStateSet == null) {
                    accessibleStateSet = new AccessibleStateSet();
                }
                if (JTable.this.getVisibleRect().intersects(JTable.this.getCellRect(this.row, this.column, false))) {
                    accessibleStateSet.add(AccessibleState.SHOWING);
                }
                else if (accessibleStateSet.contains(AccessibleState.SHOWING)) {
                    accessibleStateSet.remove(AccessibleState.SHOWING);
                }
                if (this.parent.isCellSelected(this.row, this.column)) {
                    accessibleStateSet.add(AccessibleState.SELECTED);
                }
                else if (accessibleStateSet.contains(AccessibleState.SELECTED)) {
                    accessibleStateSet.remove(AccessibleState.SELECTED);
                }
                if (this.row == JTable.this.getSelectedRow() && this.column == JTable.this.getSelectedColumn()) {
                    accessibleStateSet.add(AccessibleState.ACTIVE);
                }
                accessibleStateSet.add(AccessibleState.TRANSIENT);
                return accessibleStateSet;
            }
            
            @Override
            public Accessible getAccessibleParent() {
                return this.parent;
            }
            
            @Override
            public int getAccessibleIndexInParent() {
                return this.index;
            }
            
            @Override
            public int getAccessibleChildrenCount() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleChildrenCount();
                }
                return 0;
            }
            
            @Override
            public Accessible getAccessibleChild(int n) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    Accessible accessibleChild = currentAccessibleContext.getAccessibleChild(n);
                    currentAccessibleContext.setAccessibleParent(this);
                    return accessibleChild;
                }
                return null;
            }
            
            @Override
            public Locale getLocale() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getLocale();
                }
                return null;
            }
            
            @Override
            public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.addPropertyChangeListener(propertyChangeListener);
                }
                else {
                    super.addPropertyChangeListener(propertyChangeListener);
                }
            }
            
            @Override
            public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.removePropertyChangeListener(propertyChangeListener);
                }
                else {
                    super.removePropertyChangeListener(propertyChangeListener);
                }
            }
            
            @Override
            public AccessibleAction getAccessibleAction() {
                return this.getCurrentAccessibleContext().getAccessibleAction();
            }
            
            @Override
            public AccessibleComponent getAccessibleComponent() {
                return this;
            }
            
            @Override
            public AccessibleSelection getAccessibleSelection() {
                return this.getCurrentAccessibleContext().getAccessibleSelection();
            }
            
            @Override
            public AccessibleText getAccessibleText() {
                return this.getCurrentAccessibleContext().getAccessibleText();
            }
            
            @Override
            public AccessibleValue getAccessibleValue() {
                return this.getCurrentAccessibleContext().getAccessibleValue();
            }
            
            @Override
            public Color getBackground() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getBackground();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getBackground();
                }
                return null;
            }
            
            @Override
            public void setBackground(Color color) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setBackground(color);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setBackground(color);
                    }
                }
            }
            
            @Override
            public Color getForeground() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getForeground();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getForeground();
                }
                return null;
            }
            
            @Override
            public void setForeground(Color color) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setForeground(color);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setForeground(color);
                    }
                }
            }
            
            @Override
            public Cursor getCursor() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getCursor();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getCursor();
                }
                Accessible accessibleParent = this.getAccessibleParent();
                if ((accessibleParent instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)accessibleParent).getCursor();
                }
                return null;
            }
            
            @Override
            public void setCursor(Cursor cursor) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setCursor(cursor);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setCursor(cursor);
                    }
                }
            }
            
            @Override
            public Font getFont() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getFont();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getFont();
                }
                return null;
            }
            
            @Override
            public void setFont(Font font) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setFont(font);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setFont(font);
                    }
                }
            }
            
            @Override
            public FontMetrics getFontMetrics(Font font) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getFontMetrics(font);
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getFontMetrics(font);
                }
                return null;
            }
            
            @Override
            public boolean isEnabled() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).isEnabled();
                }
                Component currentComponent = this.getCurrentComponent();
                return currentComponent != null && currentComponent.isEnabled();
            }
            
            @Override
            public void setEnabled(boolean b) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setEnabled(b);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setEnabled(b);
                    }
                }
            }
            
            @Override
            public boolean isVisible() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).isVisible();
                }
                Component currentComponent = this.getCurrentComponent();
                return currentComponent != null && currentComponent.isVisible();
            }
            
            @Override
            public void setVisible(boolean b) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setVisible(b);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setVisible(b);
                    }
                }
            }
            
            @Override
            public boolean isShowing() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    if (currentAccessibleContext.getAccessibleParent() != null) {
                        return ((AccessibleComponent)currentAccessibleContext).isShowing();
                    }
                    return this.isVisible();
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    return currentComponent != null && currentComponent.isShowing();
                }
            }
            
            @Override
            public boolean contains(Point p) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getBounds().contains(p);
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getBounds().contains(p);
                }
                return this.getBounds().contains(p);
            }
            
            @Override
            public Point getLocationOnScreen() {
                if (this.parent != null && this.parent.isShowing()) {
                    Point locationOnScreen = this.parent.getLocationOnScreen();
                    Point location = this.getLocation();
                    location.translate(locationOnScreen.x, locationOnScreen.y);
                    return location;
                }
                return null;
            }
            
            @Override
            public Point getLocation() {
                if (this.parent != null) {
                    Rectangle cellRect = this.parent.getCellRect(this.row, this.column, false);
                    if (cellRect != null) {
                        return cellRect.getLocation();
                    }
                }
                return null;
            }
            
            @Override
            public void setLocation(Point point) {
            }
            
            @Override
            public Rectangle getBounds() {
                if (this.parent != null) {
                    return this.parent.getCellRect(this.row, this.column, false);
                }
                return null;
            }
            
            @Override
            public void setBounds(Rectangle rectangle) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setBounds(rectangle);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setBounds(rectangle);
                    }
                }
            }
            
            @Override
            public Dimension getSize() {
                if (this.parent != null) {
                    Rectangle cellRect = this.parent.getCellRect(this.row, this.column, false);
                    if (cellRect != null) {
                        return cellRect.getSize();
                    }
                }
                return null;
            }
            
            @Override
            public void setSize(Dimension dimension) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setSize(dimension);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setSize(dimension);
                    }
                }
            }
            
            @Override
            public Accessible getAccessibleAt(Point point) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getAccessibleAt(point);
                }
                return null;
            }
            
            @Override
            public boolean isFocusTraversable() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).isFocusTraversable();
                }
                Component currentComponent = this.getCurrentComponent();
                return currentComponent != null && currentComponent.isFocusTraversable();
            }
            
            @Override
            public void requestFocus() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).requestFocus();
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.requestFocus();
                    }
                }
            }
            
            @Override
            public void addFocusListener(FocusListener l) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).addFocusListener(l);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.addFocusListener(l);
                    }
                }
            }
            
            @Override
            public void removeFocusListener(FocusListener l) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).removeFocusListener(l);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.removeFocusListener(l);
                    }
                }
            }
        }
        
        private class AccessibleJTableHeaderCell extends AccessibleContext implements Accessible, AccessibleComponent
        {
            private int row;
            private int column;
            private JTableHeader parent;
            private Component rendererComponent;
            
            public AccessibleJTableHeaderCell(int row, int column, JTableHeader tableHeader, Component rendererComponent) {
                this.row = row;
                this.column = column;
                this.parent = tableHeader;
                this.rendererComponent = rendererComponent;
                this.setAccessibleParent(tableHeader);
            }
            
            @Override
            public AccessibleContext getAccessibleContext() {
                return this;
            }
            
            private AccessibleContext getCurrentAccessibleContext() {
                return this.rendererComponent.getAccessibleContext();
            }
            
            private Component getCurrentComponent() {
                return this.rendererComponent;
            }
            
            @Override
            public String getAccessibleName() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    String accessibleName = currentAccessibleContext.getAccessibleName();
                    if (accessibleName != null && accessibleName != "") {
                        return currentAccessibleContext.getAccessibleName();
                    }
                }
                if (this.accessibleName != null && this.accessibleName != "") {
                    return this.accessibleName;
                }
                return null;
            }
            
            @Override
            public void setAccessibleName(String s) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.setAccessibleName(s);
                }
                else {
                    super.setAccessibleName(s);
                }
            }
            
            @Override
            public String getAccessibleDescription() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleDescription();
                }
                return super.getAccessibleDescription();
            }
            
            @Override
            public void setAccessibleDescription(String s) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.setAccessibleDescription(s);
                }
                else {
                    super.setAccessibleDescription(s);
                }
            }
            
            @Override
            public AccessibleRole getAccessibleRole() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleRole();
                }
                return AccessibleRole.UNKNOWN;
            }
            
            @Override
            public AccessibleStateSet getAccessibleStateSet() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                Object accessibleStateSet = null;
                if (currentAccessibleContext != null) {
                    accessibleStateSet = currentAccessibleContext.getAccessibleStateSet();
                }
                if (accessibleStateSet == null) {
                    accessibleStateSet = new AccessibleStateSet();
                }
                if (JTable.this.getVisibleRect().intersects(JTable.this.getCellRect(this.row, this.column, false))) {
                    accessibleStateSet.add(AccessibleState.SHOWING);
                }
                else if (accessibleStateSet.contains(AccessibleState.SHOWING)) {
                    accessibleStateSet.remove(AccessibleState.SHOWING);
                }
                if (JTable.this.isCellSelected(this.row, this.column)) {
                    accessibleStateSet.add(AccessibleState.SELECTED);
                }
                else if (accessibleStateSet.contains(AccessibleState.SELECTED)) {
                    accessibleStateSet.remove(AccessibleState.SELECTED);
                }
                if (this.row == JTable.this.getSelectedRow() && this.column == JTable.this.getSelectedColumn()) {
                    accessibleStateSet.add(AccessibleState.ACTIVE);
                }
                accessibleStateSet.add(AccessibleState.TRANSIENT);
                return accessibleStateSet;
            }
            
            @Override
            public Accessible getAccessibleParent() {
                return this.parent;
            }
            
            @Override
            public int getAccessibleIndexInParent() {
                return this.column;
            }
            
            @Override
            public int getAccessibleChildrenCount() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleChildrenCount();
                }
                return 0;
            }
            
            @Override
            public Accessible getAccessibleChild(int n) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    Accessible accessibleChild = currentAccessibleContext.getAccessibleChild(n);
                    currentAccessibleContext.setAccessibleParent(this);
                    return accessibleChild;
                }
                return null;
            }
            
            @Override
            public Locale getLocale() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getLocale();
                }
                return null;
            }
            
            @Override
            public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.addPropertyChangeListener(propertyChangeListener);
                }
                else {
                    super.addPropertyChangeListener(propertyChangeListener);
                }
            }
            
            @Override
            public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.removePropertyChangeListener(propertyChangeListener);
                }
                else {
                    super.removePropertyChangeListener(propertyChangeListener);
                }
            }
            
            @Override
            public AccessibleAction getAccessibleAction() {
                return this.getCurrentAccessibleContext().getAccessibleAction();
            }
            
            @Override
            public AccessibleComponent getAccessibleComponent() {
                return this;
            }
            
            @Override
            public AccessibleSelection getAccessibleSelection() {
                return this.getCurrentAccessibleContext().getAccessibleSelection();
            }
            
            @Override
            public AccessibleText getAccessibleText() {
                return this.getCurrentAccessibleContext().getAccessibleText();
            }
            
            @Override
            public AccessibleValue getAccessibleValue() {
                return this.getCurrentAccessibleContext().getAccessibleValue();
            }
            
            @Override
            public Color getBackground() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getBackground();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getBackground();
                }
                return null;
            }
            
            @Override
            public void setBackground(Color color) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setBackground(color);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setBackground(color);
                    }
                }
            }
            
            @Override
            public Color getForeground() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getForeground();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getForeground();
                }
                return null;
            }
            
            @Override
            public void setForeground(Color color) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setForeground(color);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setForeground(color);
                    }
                }
            }
            
            @Override
            public Cursor getCursor() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getCursor();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getCursor();
                }
                Accessible accessibleParent = this.getAccessibleParent();
                if ((accessibleParent instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)accessibleParent).getCursor();
                }
                return null;
            }
            
            @Override
            public void setCursor(Cursor cursor) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setCursor(cursor);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setCursor(cursor);
                    }
                }
            }
            
            @Override
            public Font getFont() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getFont();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getFont();
                }
                return null;
            }
            
            @Override
            public void setFont(Font font) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setFont(font);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setFont(font);
                    }
                }
            }
            
            @Override
            public FontMetrics getFontMetrics(Font font) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getFontMetrics(font);
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getFontMetrics(font);
                }
                return null;
            }
            
            @Override
            public boolean isEnabled() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).isEnabled();
                }
                Component currentComponent = this.getCurrentComponent();
                return currentComponent != null && currentComponent.isEnabled();
            }
            
            @Override
            public void setEnabled(boolean b) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setEnabled(b);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setEnabled(b);
                    }
                }
            }
            
            @Override
            public boolean isVisible() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).isVisible();
                }
                Component currentComponent = this.getCurrentComponent();
                return currentComponent != null && currentComponent.isVisible();
            }
            
            @Override
            public void setVisible(boolean b) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setVisible(b);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setVisible(b);
                    }
                }
            }
            
            @Override
            public boolean isShowing() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    if (currentAccessibleContext.getAccessibleParent() != null) {
                        return ((AccessibleComponent)currentAccessibleContext).isShowing();
                    }
                    return this.isVisible();
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    return currentComponent != null && currentComponent.isShowing();
                }
            }
            
            @Override
            public boolean contains(Point p) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getBounds().contains(p);
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getBounds().contains(p);
                }
                return this.getBounds().contains(p);
            }
            
            @Override
            public Point getLocationOnScreen() {
                if (this.parent != null && this.parent.isShowing()) {
                    Point locationOnScreen = this.parent.getLocationOnScreen();
                    Point location = this.getLocation();
                    location.translate(locationOnScreen.x, locationOnScreen.y);
                    return location;
                }
                return null;
            }
            
            @Override
            public Point getLocation() {
                if (this.parent != null) {
                    Rectangle headerRect = this.parent.getHeaderRect(this.column);
                    if (headerRect != null) {
                        return headerRect.getLocation();
                    }
                }
                return null;
            }
            
            @Override
            public void setLocation(Point point) {
            }
            
            @Override
            public Rectangle getBounds() {
                if (this.parent != null) {
                    return this.parent.getHeaderRect(this.column);
                }
                return null;
            }
            
            @Override
            public void setBounds(Rectangle rectangle) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setBounds(rectangle);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setBounds(rectangle);
                    }
                }
            }
            
            @Override
            public Dimension getSize() {
                if (this.parent != null) {
                    Rectangle headerRect = this.parent.getHeaderRect(this.column);
                    if (headerRect != null) {
                        return headerRect.getSize();
                    }
                }
                return null;
            }
            
            @Override
            public void setSize(Dimension dimension) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setSize(dimension);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setSize(dimension);
                    }
                }
            }
            
            @Override
            public Accessible getAccessibleAt(Point point) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getAccessibleAt(point);
                }
                return null;
            }
            
            @Override
            public boolean isFocusTraversable() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).isFocusTraversable();
                }
                Component currentComponent = this.getCurrentComponent();
                return currentComponent != null && currentComponent.isFocusTraversable();
            }
            
            @Override
            public void requestFocus() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).requestFocus();
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.requestFocus();
                    }
                }
            }
            
            @Override
            public void addFocusListener(FocusListener l) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).addFocusListener(l);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.addFocusListener(l);
                    }
                }
            }
            
            @Override
            public void removeFocusListener(FocusListener l) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).removeFocusListener(l);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.removeFocusListener(l);
                    }
                }
            }
        }
    }
    
    private interface Resizable2
    {
        public abstract int getElementCount();
        
        public abstract int getLowerBoundAt(int p0);
        
        public abstract int getUpperBoundAt(int p0);
        
        public abstract void setSizeAt(int p0, int p1);
    }
    
    private interface Resizable3 extends JTable.Resizable2
    {
        public abstract int getMidPointAt(int p0);
    }
}
